/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bindinterface.h"
#include "audioengine.h"
#include "audioport.h"
#include "basicclock.h"
#include "beattracker.h"
#include "bindaudio.h"
#include "bindio.h"
#include "bindjack.h"
#include "bindlv2.h"
#include "bindmidi.h"
#include "bindosc.h"
#include "bindscript.h"
#include "bindtime.h"
#include "io.h"
#include "jackclock.h"
#include "link.h"
#include "lv2plugin.h"
#include "midibeatclock.h"
#include "midifile.h"
#include "midiport.h"
#include "mixer.h"
#include "onsetdetector.h"
#include "timebase.h"
#include "variableclock.h"
#include "zipfile.h"

namespace bipscript {


namespace audio { 
namespace binding {
audio::Source *getSource(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &InputObject, false))) {
        return static_cast<audio::AudioInputPort *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &MixerObject, false))) {
        return static_cast<audio::Mixer *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &MultiInputObject, false))) {
        return static_cast<audio::MultiInput *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &StereoInputObject, false))) {
        return static_cast<audio::StereoInput *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &lv2::binding::PluginObject, false))) {
        return static_cast<lv2::Plugin *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, nullptr, false))) {
        return resolve_source(ptr);
    }
	return nullptr;
}

audio::Source *getSource(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &InputObject) {
        return static_cast<audio::AudioInputPort *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &MixerObject) {
        return static_cast<audio::Mixer *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &MultiInputObject) {
        return static_cast<audio::MultiInput *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &StereoInputObject) {
        return static_cast<audio::StereoInput *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &lv2::binding::PluginObject) {
        return static_cast<lv2::Plugin *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}

namespace audio { 
namespace binding {
audio::Sink *getSink(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &MixerObject, false))) {
        return static_cast<audio::Mixer *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &MultiOutputObject, false))) {
        return static_cast<audio::MultiOutput *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &OnsetDetectorObject, false))) {
        return static_cast<audio::OnsetDetector *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &OutputObject, false))) {
        return static_cast<audio::Output *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &StereoOutputObject, false))) {
        return static_cast<audio::StereoOutput *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &lv2::binding::PluginObject, false))) {
        return static_cast<lv2::Plugin *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, nullptr, false))) {
        return resolve_sink(ptr);
    }
	return nullptr;
}

audio::Sink *getSink(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &MixerObject) {
        return static_cast<audio::Mixer *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &MultiOutputObject) {
        return static_cast<audio::MultiOutput *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &OnsetDetectorObject) {
        return static_cast<audio::OnsetDetector *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &OutputObject) {
        return static_cast<audio::Output *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &StereoOutputObject) {
        return static_cast<audio::StereoOutput *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &lv2::binding::PluginObject) {
        return static_cast<lv2::Plugin *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}

namespace io { 
namespace binding {
io::Path *getPath(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &FileObject, false))) {
        return static_cast<io::File *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &FolderObject, false))) {
        return static_cast<io::Folder *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &ZipFileObject, false))) {
        return static_cast<io::ZipFile *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &ZipFolderObject, false))) {
        return static_cast<io::ZipFolder *>(ptr);
    }
	return nullptr;
}

io::Path *getPath(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &FileObject) {
        return static_cast<io::File *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &FolderObject) {
        return static_cast<io::Folder *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &ZipFileObject) {
        return static_cast<io::ZipFile *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &ZipFolderObject) {
        return static_cast<io::ZipFolder *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}

namespace midi { 
namespace binding {
midi::Message *getMessage(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &ControlObject, false))) {
        return static_cast<midi::Control *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &CopyrightObject, false))) {
        return static_cast<midi::Copyright *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &CuePointObject, false))) {
        return static_cast<midi::CuePoint *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &InstrumentObject, false))) {
        return static_cast<midi::Instrument *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &KeySignatureObject, false))) {
        return static_cast<midi::KeySignature *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &LyricObject, false))) {
        return static_cast<midi::Lyric *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &MarkerObject, false))) {
        return static_cast<midi::Marker *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &NoteOffObject, false))) {
        return static_cast<midi::NoteOff *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &NoteOnObject, false))) {
        return static_cast<midi::NoteOn *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &PitchBendObject, false))) {
        return static_cast<midi::PitchBend *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &ProgramChangeObject, false))) {
        return static_cast<midi::ProgramChange *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &SysexObject, false))) {
        return static_cast<midi::Sysex *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &TempoChangeObject, false))) {
        return static_cast<midi::TempoChange *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &TextObject, false))) {
        return static_cast<midi::Text *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &TimeSignatureObject, false))) {
        return static_cast<midi::TimeSignature *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &TrackNameObject, false))) {
        return static_cast<midi::TrackName *>(ptr);
    }
	return nullptr;
}

midi::Message *getMessage(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &ControlObject) {
        return static_cast<midi::Control *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &CopyrightObject) {
        return static_cast<midi::Copyright *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &CuePointObject) {
        return static_cast<midi::CuePoint *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &InstrumentObject) {
        return static_cast<midi::Instrument *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &KeySignatureObject) {
        return static_cast<midi::KeySignature *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &LyricObject) {
        return static_cast<midi::Lyric *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &MarkerObject) {
        return static_cast<midi::Marker *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &NoteOffObject) {
        return static_cast<midi::NoteOff *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &NoteOnObject) {
        return static_cast<midi::NoteOn *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &PitchBendObject) {
        return static_cast<midi::PitchBend *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &ProgramChangeObject) {
        return static_cast<midi::ProgramChange *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &SysexObject) {
        return static_cast<midi::Sysex *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &TempoChangeObject) {
        return static_cast<midi::TempoChange *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &TextObject) {
        return static_cast<midi::Text *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &TimeSignatureObject) {
        return static_cast<midi::TimeSignature *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &TrackNameObject) {
        return static_cast<midi::TrackName *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}

namespace midi { 
namespace binding {
midi::Source *getSource(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &lv2::binding::PluginObject, false))) {
        return static_cast<lv2::Plugin *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &ClockGeneratorObject, false))) {
        return static_cast<midi::ClockGenerator *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &InputObject, false))) {
        return static_cast<midi::MidiInputPort *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, nullptr, false))) {
        return resolve_source(ptr);
    }
	return nullptr;
}

midi::Source *getSource(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &lv2::binding::PluginObject) {
        return static_cast<lv2::Plugin *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &ClockGeneratorObject) {
        return static_cast<midi::ClockGenerator *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &InputObject) {
        return static_cast<midi::MidiInputPort *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}

namespace midi { 
namespace binding {
midi::Sink *getSink(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &lv2::binding::PluginObject, false))) {
        return static_cast<lv2::Plugin *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &OutputObject, false))) {
        return static_cast<midi::MidiOutputPort *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, nullptr, false))) {
        return resolve_sink(ptr);
    }
	return nullptr;
}

midi::Sink *getSink(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &lv2::binding::PluginObject) {
        return static_cast<lv2::Plugin *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &OutputObject) {
        return static_cast<midi::MidiOutputPort *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}

namespace time { 
namespace binding {
time::Clock *getClock(HSQUIRRELVM &vm, int index) {
    SQUserPointer ptr;
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &audio::binding::BeatTrackerObject, false))) {
        return static_cast<audio::BeatTracker *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &jack::binding::ClockObject, false))) {
        return static_cast<jack::Clock *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &jack::binding::TimebaseObject, false))) {
        return static_cast<jack::Timebase *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &midi::binding::BeatTrackerObject, false))) {
        return static_cast<midi::BeatTracker *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &BasicClockObject, false))) {
        return static_cast<time::BasicClock *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &LinkObject, false))) {
        return static_cast<time::Link *>(ptr);
    }
    if (!SQ_FAILED(sq_getinstanceup(vm, index, &ptr, &VariableClockObject, false))) {
        return static_cast<time::VariableClock *>(ptr);
    }
	return nullptr;
}

time::Clock *getClock(SQUserPointer type, HSQOBJECT *arg) {
    if(type == &audio::binding::BeatTrackerObject) {
        return static_cast<audio::BeatTracker *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &jack::binding::ClockObject) {
        return static_cast<jack::Clock *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &jack::binding::TimebaseObject) {
        return static_cast<jack::Timebase *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &midi::binding::BeatTrackerObject) {
        return static_cast<midi::BeatTracker *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &BasicClockObject) {
        return static_cast<time::BasicClock *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &LinkObject) {
        return static_cast<time::Link *>(bip_getinstanceuserpointer(arg));
    }
    if(type == &VariableClockObject) {
        return static_cast<time::VariableClock *>(bip_getinstanceuserpointer(arg));
    }
    return nullptr;
}

}}
namespace io { 
namespace binding {
void pushPath(HSQUIRRELVM &vm, io::Path *obj) {
    if (io::File* ret = dynamic_cast<io::File *>(obj)) {
        sq_pushobject(vm, FileObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
        sq_setreleasehook(vm, -1, BindUtil::Delete<io::File>);
    }
    if (io::Folder* ret = dynamic_cast<io::Folder *>(obj)) {
        sq_pushobject(vm, FolderObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
        sq_setreleasehook(vm, -1, BindUtil::Delete<io::Folder>);
    }
    if (io::ZipFile* ret = dynamic_cast<io::ZipFile *>(obj)) {
        sq_pushobject(vm, ZipFileObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
        sq_setreleasehook(vm, -1, BindUtil::Delete<io::ZipFile>);
    }
    if (io::ZipFolder* ret = dynamic_cast<io::ZipFolder *>(obj)) {
        sq_pushobject(vm, ZipFolderObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
        sq_setreleasehook(vm, -1, BindUtil::Delete<io::ZipFolder>);
    }
}
}}
namespace midi { 
namespace binding {
void pushMessage(HSQUIRRELVM &vm, midi::Message *obj) {
    if (midi::Control* ret = dynamic_cast<midi::Control *>(obj)) {
        sq_pushobject(vm, ControlObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Control>);
    }
    if (midi::Copyright* ret = dynamic_cast<midi::Copyright *>(obj)) {
        sq_pushobject(vm, CopyrightObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Copyright>);
    }
    if (midi::CuePoint* ret = dynamic_cast<midi::CuePoint *>(obj)) {
        sq_pushobject(vm, CuePointObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::CuePoint>);
    }
    if (midi::Instrument* ret = dynamic_cast<midi::Instrument *>(obj)) {
        sq_pushobject(vm, InstrumentObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Instrument>);
    }
    if (midi::KeySignature* ret = dynamic_cast<midi::KeySignature *>(obj)) {
        sq_pushobject(vm, KeySignatureObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::KeySignature>);
    }
    if (midi::Lyric* ret = dynamic_cast<midi::Lyric *>(obj)) {
        sq_pushobject(vm, LyricObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Lyric>);
    }
    if (midi::Marker* ret = dynamic_cast<midi::Marker *>(obj)) {
        sq_pushobject(vm, MarkerObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Marker>);
    }
    if (midi::NoteOff* ret = dynamic_cast<midi::NoteOff *>(obj)) {
        sq_pushobject(vm, NoteOffObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::NoteOff>);
    }
    if (midi::NoteOn* ret = dynamic_cast<midi::NoteOn *>(obj)) {
        sq_pushobject(vm, NoteOnObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::NoteOn>);
    }
    if (midi::PitchBend* ret = dynamic_cast<midi::PitchBend *>(obj)) {
        sq_pushobject(vm, PitchBendObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::PitchBend>);
    }
    if (midi::ProgramChange* ret = dynamic_cast<midi::ProgramChange *>(obj)) {
        sq_pushobject(vm, ProgramChangeObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::ProgramChange>);
    }
    if (midi::Sysex* ret = dynamic_cast<midi::Sysex *>(obj)) {
        sq_pushobject(vm, SysexObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Sysex>);
    }
    if (midi::TempoChange* ret = dynamic_cast<midi::TempoChange *>(obj)) {
        sq_pushobject(vm, TempoChangeObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::TempoChange>);
    }
    if (midi::Text* ret = dynamic_cast<midi::Text *>(obj)) {
        sq_pushobject(vm, TextObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Text>);
    }
    if (midi::TimeSignature* ret = dynamic_cast<midi::TimeSignature *>(obj)) {
        sq_pushobject(vm, TimeSignatureObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::TimeSignature>);
    }
    if (midi::TrackName* ret = dynamic_cast<midi::TrackName *>(obj)) {
        sq_pushobject(vm, TrackNameObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::TrackName>);
    }
}
}}
namespace time { 
namespace binding {
void pushClock(HSQUIRRELVM &vm, time::Clock *obj) {
    if (audio::BeatTracker* ret = dynamic_cast<audio::BeatTracker *>(obj)) {
        sq_pushobject(vm, audio::binding::BeatTrackerObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
    if (jack::Clock* ret = dynamic_cast<jack::Clock *>(obj)) {
        sq_pushobject(vm, jack::binding::ClockObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
    if (jack::Timebase* ret = dynamic_cast<jack::Timebase *>(obj)) {
        sq_pushobject(vm, jack::binding::TimebaseObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
    if (midi::BeatTracker* ret = dynamic_cast<midi::BeatTracker *>(obj)) {
        sq_pushobject(vm, midi::binding::BeatTrackerObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
    if (time::BasicClock* ret = dynamic_cast<time::BasicClock *>(obj)) {
        sq_pushobject(vm, BasicClockObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
    if (time::Link* ret = dynamic_cast<time::Link *>(obj)) {
        sq_pushobject(vm, LinkObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
    if (time::VariableClock* ret = dynamic_cast<time::VariableClock *>(obj)) {
        sq_pushobject(vm, VariableClockObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret);
    }
}
}}

}
