/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BINDINTERFACE_H
#define BINDINTERFACE_H

#include "bipsquirrel.h"

namespace bipscript {

namespace audio { class Source; }
namespace audio { class Sink; }
namespace io { class Path; }
namespace midi { class Message; }
namespace midi { class Source; }
namespace midi { class Sink; }
namespace time { class Clock; }

// argument type conversion methods
namespace audio { 
namespace binding {
    audio::Source *getSource(HSQUIRRELVM &vm, int index);
    audio::Source *getSource(SQUserPointer type, HSQOBJECT *arg);
}}
namespace audio { 
namespace binding {
    audio::Sink *getSink(HSQUIRRELVM &vm, int index);
    audio::Sink *getSink(SQUserPointer type, HSQOBJECT *arg);
}}
namespace io { 
namespace binding {
    io::Path *getPath(HSQUIRRELVM &vm, int index);
    io::Path *getPath(SQUserPointer type, HSQOBJECT *arg);
}}
namespace midi { 
namespace binding {
    midi::Message *getMessage(HSQUIRRELVM &vm, int index);
    midi::Message *getMessage(SQUserPointer type, HSQOBJECT *arg);
}}
namespace midi { 
namespace binding {
    midi::Source *getSource(HSQUIRRELVM &vm, int index);
    midi::Source *getSource(SQUserPointer type, HSQOBJECT *arg);
}}
namespace midi { 
namespace binding {
    midi::Sink *getSink(HSQUIRRELVM &vm, int index);
    midi::Sink *getSink(SQUserPointer type, HSQOBJECT *arg);
}}
namespace time { 
namespace binding {
    time::Clock *getClock(HSQUIRRELVM &vm, int index);
    time::Clock *getClock(SQUserPointer type, HSQOBJECT *arg);
}}

// return type conversion methods 
namespace io { 
namespace binding {
    void pushPath(HSQUIRRELVM &vm, io::Path *obj);
}}
namespace midi { 
namespace binding {
    void pushMessage(HSQUIRRELVM &vm, midi::Message *obj);
}}
namespace time { 
namespace binding {
    void pushClock(HSQUIRRELVM &vm, time::Clock *obj);
}}

// resolve methods - implemented elsewhere
namespace audio { 
namespace binding {
    audio::Source *resolve_source(SQUserPointer obj);
}}
namespace audio { 
namespace binding {
    audio::Sink *resolve_sink(SQUserPointer obj);
}}
namespace midi { 
namespace binding {
    midi::Source *resolve_source(SQUserPointer obj);
}}
namespace midi { 
namespace binding {
    midi::Sink *resolve_sink(SQUserPointer obj);
}}
}


#endif // BINDINTERFACE_H
