/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "bindjack.h"
#include "bindinterface.h"
#include "bindutil.h"
#include "bindtime.h"

#include "jackclock.h"
#include "timebase.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

namespace jack {

namespace binding {

// object references to types in this package
HSQOBJECT ClockObject;
HSQOBJECT PositionObject;
HSQOBJECT TimebaseObject;

#define BIND_CLASS_NAME "Jack"
//
// Jack getPorts
//
#define BIND_METHOD_NAME "getPorts"
SQInteger JackgetPorts(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_NULLABLE_STRING(name, 1)
    // get parameter 2 "type" as string
    BIND_GET_CHECK_NULLABLE_STRING(type, 2)
    // get parameter 3 "flags" as integer
    BIND_GET_CHECK_INTEGER(flags, 3)
    // return value
    HSQOBJECT ret;
    // call the implementation
    BIND_WRAP_TRY(ret = AudioEngine::instance().getPorts(name, type, flags))
    // push return value
    sq_pushobject(vm, ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Jack onXrun
//
#define BIND_METHOD_NAME "onXrun"
SQInteger JackonXrun(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(AudioEngine::instance().onXrun(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

#undef BIND_CLASS_NAME
//
// Jack.Clock class
//
#define BIND_CLASS_NAME "Clock"
#define BIND_METHOD_NAME "constructor"
SQInteger ClockCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Clock *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = ClockFactory::instance().newClock())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger ClockNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Jack.Clock");
}
//
// Jack.Clock property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger ClockGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(jack::Clock)
    // dispatch appropriate getter
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Jack.Clock property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger ClockSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(jack::Clock)
    // dispatch appropriate setter
    if(!strcmp(key, "tempo")) {
        return sq_throwerror(vm, "property \"tempo\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Jack.Clock start
//
#define BIND_METHOD_NAME "start"
SQInteger Clockstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Jack.Clock stop
//
#define BIND_METHOD_NAME "stop"
SQInteger Clockstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clock)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger ClockToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Jack.Clock)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Jack.Position class
//
#define BIND_CLASS_NAME "Position"
SQInteger PositionCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Jack.Position");
}

SQInteger PositionNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Jack.Position");
}
//
// Jack.Position property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger PositionGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(jack::Position)
    // dispatch appropriate getter
    if(!strcmp(key, "bar")) {
        SQInteger ret = obj->bar();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "beat")) {
        SQInteger ret = obj->beat();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "measure")) {
        SQFloat ret = obj->measure();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    if(!strcmp(key, "tick")) {
        SQInteger ret = obj->tick();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Jack.Position property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger PositionSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(jack::Position)
    // dispatch appropriate setter
    if(!strcmp(key, "bar")) {
        return sq_throwerror(vm, "property \"bar\" is read-only");
    }
    if(!strcmp(key, "beat")) {
        return sq_throwerror(vm, "property \"beat\" is read-only");
    }
    if(!strcmp(key, "measure")) {
        return sq_throwerror(vm, "property \"measure\" is read-only");
    }
    if(!strcmp(key, "tick")) {
        return sq_throwerror(vm, "property \"tick\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger PositionToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Jack.Position)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Jack.Timebase class
//
#define BIND_CLASS_NAME "Timebase"
#define BIND_METHOD_NAME "constructor"
SQInteger TimebaseCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Timebase *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = TimebaseCache::instance().newTimebase(bpm))
    } else {
        // get parameter 2 "time" as Time.Signature
        BIND_GET_CHECK_OBJECT(time, 2, Time, Signature, time, Signature)
        // call the implementation
        BIND_WRAP_TRY(obj = TimebaseCache::instance().newTimebase(bpm, *time))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger TimebaseNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Jack.Timebase");
}
//
// Jack.Timebase property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TimebaseGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(jack::Timebase)
    // dispatch appropriate getter
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Jack.Timebase property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TimebaseSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(jack::Timebase)
    // dispatch appropriate setter
    if(!strcmp(key, "tempo")) {
        // get parameter 2 "tempo" as float
        BIND_GET_CHECK_FLOAT(tempo, 2)
        BIND_WRAP_TRY(obj->tempo(tempo))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Jack.Timebase onSync
//
#define BIND_METHOD_NAME "onSync"
SQInteger TimebaseonSync(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Timebase)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj->onSync(handler))
    } else {
        // get parameter 2 "wait" as bool
        BIND_GET_CHECK_BOOL(wait, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->onSync(handler, wait))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Jack.Timebase reposition
//
#define BIND_METHOD_NAME "reposition"
SQInteger Timebasereposition(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Timebase)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "location" as float
    BIND_GET_CHECK_FLOAT(location, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->reposition(location))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Jack.Timebase schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Timebaseschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Timebase)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(BIND_CHECK_FLOAT(1)) {
        // get parameter 1 "tempo" as float
        BIND_GET_FLOAT(tempo, 1)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->schedule(tempo, measure))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &time::binding::SignatureObject) {
        // get parameter 1 "signature" as Time.Signature
        BIND_GET_OBJECT(signature, 1, Time, Signature, time, Signature)
        // get parameter 2 "bar" as integer
        BIND_GET_CHECK_INTEGER(bar, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->schedule(*signature, bar))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Time.Signature, float}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Time.Signature, float}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Jack.Timebase start
//
#define BIND_METHOD_NAME "start"
SQInteger Timebasestart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Timebase)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Jack.Timebase stop
//
#define BIND_METHOD_NAME "stop"
SQInteger Timebasestop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Timebase)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger TimebaseToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Jack.Timebase)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

SQInteger JackNewSlot(HSQUIRRELVM vm)
{
	return sq_throwerror(vm, "package table does not support the new slot operator");
}

void bindJack(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Jack", 4);

    // create enumeration
    sq_pushstring(vm, "PortFlag", -1);
    sq_newtable(vm);
    sq_pushstring(vm, "IsInput", -1);
    sq_pushinteger(vm, JackPortIsInput);
    sq_newslot(vm, -3, true);
    sq_pushstring(vm, "IsOutput", -1);
    sq_pushinteger(vm, JackPortIsOutput);
    sq_newslot(vm, -3, true);
    sq_newslot(vm, -3, true);
    // static methods
    binding.addMethod("onXrun", 6, &JackonXrun);
    binding.addMethod("getPorts", 8, &JackgetPorts);

    // create class Jack.Clock
    binding.startClass("Clock", 5, ClockObject);
    binding.addMethod("constructor", 11, &ClockCtor); // ctor
    binding.addMethod("_cloned", 7, &ClockNoclone); // unclonable
    binding.addMethod("_get", 4, &ClockGetters);
    binding.addMethod("_set", 4, &ClockSetters);
    binding.addMethod("start", 5, &Clockstart);
    binding.addMethod("stop", 4, &Clockstop);
    binding.addMethod("_tostring", 9, &ClockToString);
    binding.endClass();

    // create class Jack.Position
    binding.startClass("Position", 8, PositionObject);
    binding.addMethod("constructor", 11, &PositionCtor); // ctor
    binding.addMethod("_cloned", 7, &PositionNoclone); // unclonable
    binding.addMethod("_get", 4, &PositionGetters);
    binding.addMethod("_set", 4, &PositionSetters);
    binding.addMethod("_tostring", 9, &PositionToString);
    binding.endClass();

    // create class Jack.Timebase
    binding.startClass("Timebase", 8, TimebaseObject);
    binding.addMethod("constructor", 11, &TimebaseCtor); // ctor
    binding.addMethod("_cloned", 7, &TimebaseNoclone); // unclonable
    binding.addMethod("_get", 4, &TimebaseGetters);
    binding.addMethod("_set", 4, &TimebaseSetters);
    binding.addMethod("onSync", 6, &TimebaseonSync);
    binding.addMethod("reposition", 10, &Timebasereposition);
    binding.addMethod("schedule", 8, &Timebaseschedule);
    binding.addMethod("start", 5, &Timebasestart);
    binding.addMethod("stop", 4, &Timebasestop);
    binding.addMethod("_tostring", 9, &TimebaseToString);
    binding.endClass();

    sq_newtable(vm); // delegate table for package properties
    binding.addMethod("_newslot", 8, &JackNewSlot);
    sq_setdelegate(vm, -2);

    binding.endTable(); // end package table
}
}}}
