/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "bindlv2.h"
#include "bindinterface.h"
#include "bindutil.h"
#include "bindaudio.h"
#include "bindio.h"

#include "lv2plugin.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

namespace lv2 {

namespace binding {

// object references to types in this package
HSQOBJECT ControlChangeObject;
HSQOBJECT PluginObject;
HSQOBJECT StateObject;

//
// Lv2.ControlChange class
//
#define BIND_CLASS_NAME "ControlChange"
#define BIND_METHOD_NAME "constructor"
SQInteger ControlChangeCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    ControlChange *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "port" as string
    BIND_GET_CHECK_STRING(port, 1)
    // get parameter 2 "value" as float
    BIND_GET_CHECK_FLOAT(value, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = new ControlChange(port, value))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<ControlChange>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Lv2.ControlChange property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger ControlChangeGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(lv2::ControlChange)
    // dispatch appropriate getter
    if(!strcmp(key, "port")) {
        std::string ret = obj->port();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    if(!strcmp(key, "value")) {
        SQFloat ret = obj->value();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Lv2.ControlChange property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger ControlChangeSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(lv2::ControlChange)
    // dispatch appropriate setter
    if(!strcmp(key, "port")) {
        return sq_throwerror(vm, "property \"port\" is read-only");
    }
    if(!strcmp(key, "value")) {
        return sq_throwerror(vm, "property \"value\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger ControlChangeToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Lv2.ControlChange)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Lv2.Plugin class
//
#define BIND_CLASS_NAME "Plugin"
#define BIND_METHOD_NAME "constructor"
SQInteger PluginCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Plugin *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        BIND_NARGS_CHECK_MAX(2) // check max parameter count
        // get parameter 1 "uri" as string
        BIND_GET_STRING(uri, 1)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj = lv2::PluginFactory::instance().newPlugin(uri))
        } else {
            HSQOBJECT arg2;
            sq_getstackobj(vm, 3, &arg2);
            if(arg2._type == OT_STRING) {
                // get parameter 2 "preset" as string
                BIND_GET_STRING(preset, 2)
                // call the implementation
                BIND_WRAP_TRY(obj = lv2::PluginFactory::instance().newPlugin(uri, preset))
            }
            else if(arg2._type == OT_INSTANCE) {
            SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
            if(arg2type == &StateObject) {
                // get parameter 2 "state" as Lv2.State
                BIND_GET_OBJECT(state, 2, Lv2, State, lv2, State)
                // call the implementation
                BIND_WRAP_TRY(obj = lv2::PluginFactory::instance().newPlugin(uri, *state))
            }
            else {
                return sq_throwerror(vm, "argument 2 is not of type {string, Lv2.State}");
            }
            }
            else {
                return sq_throwerror(vm, "argument 2 is not of type {string, Lv2.State}");
            }
        }
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &io::binding::FileObject) {
        BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
        // get parameter 1 "statefile" as IO.File
        BIND_GET_OBJECT(statefile, 1, IO, File, io, File)
        // call the implementation
        BIND_WRAP_TRY(obj = lv2::PluginFactory::instance().newPlugin(*statefile))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, IO.File}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, IO.File}");
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger PluginNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Lv2.Plugin");
}
//
// Lv2.Plugin property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger PluginGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(lv2::Plugin)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "midiin")) {
        HSQOBJECT ret = obj->midiin();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "midiout")) {
        HSQOBJECT ret = obj->midiout();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger PluginSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(lv2::Plugin)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    if(!strcmp(key, "midiin")) {
        return sq_throwerror(vm, "property \"midiin\" is read-only");
    }
    if(!strcmp(key, "midiout")) {
        return sq_throwerror(vm, "property \"midiout\" is read-only");
    }
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin clear
//
#define BIND_METHOD_NAME "clear"
SQInteger Pluginclear(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->clear())
    } else {
        // get parameter 1 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 1, Time, Clock, time, Clock)
        // call the implementation
        BIND_WRAP_TRY(obj->clear(*clock))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin control
//
#define BIND_METHOD_NAME "control"
SQInteger Plugincontrol(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "control" as string
    BIND_GET_CHECK_STRING(control, 1)
    // get parameter 2 "value" as float
    BIND_GET_CHECK_FLOAT(value, 2)
    // call the implementation
    BIND_WRAP_TRY(obj->control(control, value))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin grab
//
#define BIND_METHOD_NAME "grab"
SQInteger Plugingrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin onWorkResponse
//
#define BIND_METHOD_NAME "onWorkResponse"
SQInteger PluginonWorkResponse(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onWorkResponse(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin play
//
#define BIND_METHOD_NAME "play"
SQInteger Pluginplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &audio::binding::BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &audio::binding::ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin preset
//
#define BIND_METHOD_NAME "preset"
SQInteger Pluginpreset(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->preset(name))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin remove
//
#define BIND_METHOD_NAME "remove"
SQInteger Pluginremove(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &midi::binding::MessageEventObject) {
        // get parameter 1 "event" as Midi.MessageEvent
        BIND_GET_OBJECT(event, 1, Midi, MessageEvent, midi, MessageEvent)
        // call the implementation
        BIND_WRAP_TRY(obj->remove(*event))
    }
    else if(arg1type == &midi::binding::NoteEventObject) {
        // get parameter 1 "event" as Midi.NoteEvent
        BIND_GET_OBJECT(event, 1, Midi, NoteEvent, midi, NoteEvent)
        // call the implementation
        BIND_WRAP_TRY(obj->remove(*event))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.MessageEvent, Midi.NoteEvent}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.MessageEvent, Midi.NoteEvent}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin saveState
//
#define BIND_METHOD_NAME "saveState"
SQInteger PluginsaveState(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "path" as string
    BIND_GET_CHECK_STRING(path, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->saveState(path))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Pluginschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &ControlChangeObject) {
        // get parameter 1 "change" as Lv2.ControlChange
        BIND_GET_OBJECT(change, 1, Lv2, ControlChange, lv2, ControlChange)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*change, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*change, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else if(arg1type == &audio::binding::BufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio, Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*buffer, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else if(arg1type == &audio::binding::ClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio, Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*clip, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else if(arg1type == &midi::binding::NoteObject) {
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::NoteEvent* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->midi::Sink::schedule(*note, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->midi::Sink::schedule(*note, measure, *clock))
        }
        // push return value
        BIND_PUSH_OBJECT(ret->addref(), NoteEvent, midi, NoteEvent, Decrement)
        return 1;

    }
    else if(arg1type == &midi::binding::PatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->midi::Sink::schedule(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->midi::Sink::schedule(*pattern, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else if(midi::Message *message = midi::binding::getMessage(arg1type, &arg1)) {
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::MessageEvent* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->midi::Sink::schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->midi::Sink::schedule(*message, measure, *clock))
        }
        // push return value
        BIND_PUSH_OBJECT(ret->addref(), MessageEvent, midi, MessageEvent, Decrement)
        return 1;

    }
    else if(arg1type == &midi::binding::SequenceObject) {
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi, Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::Sequence* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->midi::Sink::schedule(*sequence, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->midi::Sink::schedule(*sequence, measure, *clock))
        }
        // push return value
        BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
        return 1;

    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Lv2.ControlChange, Audio.Buffer, Audio.Clip, Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Lv2.ControlChange, Audio.Buffer, Audio.Clip, Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }}
#undef BIND_METHOD_NAME

//
// Lv2.Plugin send
//
#define BIND_METHOD_NAME "send"
SQInteger Pluginsend(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Plugin)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(midi::Message *message = midi::binding::getMessage(arg1type, &arg1)) {
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*message))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            if(numargs == 3) {
                // call the implementation
                BIND_WRAP_TRY(obj->send(*message, measure))
            } else {
                // get parameter 3 "clock" as Time.Clock
                BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
                // call the implementation
                BIND_WRAP_TRY(obj->send(*message, measure, *clock))
            }
        }
    }
    else if(arg1type == &midi::binding::NoteObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*note, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*note, measure, *clock))
        }
    }
    else if(arg1type == &midi::binding::PatternObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*pattern, measure, *clock))
        }
    }
    else if(arg1type == &midi::binding::SequenceObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi, Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*sequence, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*sequence, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger PluginToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Lv2.Plugin)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Lv2.State class
//
#define BIND_CLASS_NAME "State"
#define BIND_METHOD_NAME "constructor"
SQInteger StateCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    State *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_TABLE) {
        // get parameter 1 "state" as table
        BIND_GET_TABLE(state, 1)
        // call the implementation
        BIND_WRAP_TRY(obj = new State(state))
    }
    else if(arg1._type == OT_STRING) {
        // get parameter 1 "file" as string
        BIND_GET_STRING(file, 1)
        // call the implementation
        BIND_WRAP_TRY(obj = new State(file))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {table, string}");
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<State>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger StateToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Lv2.State)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

SQInteger Lv2NewSlot(HSQUIRRELVM vm)
{
	return sq_throwerror(vm, "package table does not support the new slot operator");
}

void bindLv2(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Lv2", 3);

    // create class Lv2.ControlChange
    binding.startClass("ControlChange", 13, ControlChangeObject);
    binding.addMethod("constructor", 11, &ControlChangeCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<ControlChange>); // clone
    binding.addMethod("_get", 4, &ControlChangeGetters);
    binding.addMethod("_set", 4, &ControlChangeSetters);
    binding.addMethod("_tostring", 9, &ControlChangeToString);
    binding.endClass();

    // create class Lv2.Plugin
    binding.startClass("Plugin", 6, PluginObject);
    binding.addMethod("constructor", 11, &PluginCtor); // ctor
    binding.addMethod("_cloned", 7, &PluginNoclone); // unclonable
    binding.addMethod("_get", 4, &PluginGetters);
    binding.addMethod("_set", 4, &PluginSetters);
    binding.addMethod("clear", 5, &Pluginclear);
    binding.addMethod("control", 7, &Plugincontrol);
    binding.addMethod("grab", 4, &Plugingrab);
    binding.addMethod("onWorkResponse", 14, &PluginonWorkResponse);
    binding.addMethod("play", 4, &Pluginplay);
    binding.addMethod("preset", 6, &Pluginpreset);
    binding.addMethod("remove", 6, &Pluginremove);
    binding.addMethod("saveState", 9, &PluginsaveState);
    binding.addMethod("schedule", 8, &Pluginschedule);
    binding.addMethod("send", 4, &Pluginsend);
    binding.addMethod("_tostring", 9, &PluginToString);
    binding.endClass();

    // create class Lv2.State
    binding.startClass("State", 5, StateObject);
    binding.addMethod("constructor", 11, &StateCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<State>); // clone
    binding.addMethod("_tostring", 9, &StateToString);
    binding.endClass();

    sq_newtable(vm); // delegate table for package properties
    binding.addMethod("_newslot", 8, &Lv2NewSlot);
    sq_setdelegate(vm, -2);

    binding.endTable(); // end package table
}
}}}
