/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "bindmidi.h"
#include "bindinterface.h"
#include "bindutil.h"
#include "bindtime.h"
#include "bindio.h"

#include "abc.h"
#include "beattracker.h"
#include "midibeatclock.h"
#include "midifile.h"
#include "midiport.h"
#include "mml.h"
#include "module_midi.h"
#include "midisequence.h"
#include "tab.h"
#include "midipackage.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

namespace midi {

namespace binding {

// object references to types in this package
HSQOBJECT AbcObject;
HSQOBJECT BeatTrackerObject;
HSQOBJECT ClockGeneratorObject;
HSQOBJECT ConnectionObject;
HSQOBJECT ConnectorObject;
HSQOBJECT ControlObject;
HSQOBJECT CopyrightObject;
HSQOBJECT CuePointObject;
HSQOBJECT FileObject;
HSQOBJECT InputObject;
HSQOBJECT InstrumentObject;
HSQOBJECT KeySignatureObject;
HSQOBJECT LyricObject;
HSQOBJECT MMLObject;
HSQOBJECT MarkerObject;
HSQOBJECT MessageEventObject;
HSQOBJECT NoteObject;
HSQOBJECT NoteEventObject;
HSQOBJECT NoteOffObject;
HSQOBJECT NoteOnObject;
HSQOBJECT OutputObject;
HSQOBJECT PatternObject;
HSQOBJECT PitchBendObject;
HSQOBJECT ProgramChangeObject;
HSQOBJECT SequenceObject;
HSQOBJECT SysexObject;
HSQOBJECT TabObject;
HSQOBJECT TempoChangeObject;
HSQOBJECT TextObject;
HSQOBJECT TimeSignatureObject;
HSQOBJECT TrackNameObject;

#define BIND_CLASS_NAME "Midi"
//
// Midi abc
//
#define BIND_METHOD_NAME "abc"
SQInteger Midiabc(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "abc" as string
    BIND_GET_CHECK_STRING(abc, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Midi::abc(abc))
    // push return value
    BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

#undef BIND_CLASS_NAME
//
// Midi.Abc class
//
#define BIND_CLASS_NAME "Abc"
#define BIND_METHOD_NAME "constructor"
SQInteger AbcCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Abc *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Abc())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Abc>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Abc abc
//
#define BIND_METHOD_NAME "abc"
SQInteger Abcabc(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Abc)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "note" as Midi.Note
    BIND_GET_CHECK_OBJECT(note, 1, Midi, Note, midi, Note)
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->abc(*note))
    // push return value
    BIND_PUSH_STRING(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Abc file
//
#define BIND_METHOD_NAME "file"
SQInteger Abcfile(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Abc)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "abc" as string
    BIND_GET_CHECK_STRING(abc, 1)
    // return value
    midi::File* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->file(abc))
    // push return value
    BIND_PUSH_OBJECT(ret, File, midi, File, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Abc sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger Abcsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Abc)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "abc" as string
    BIND_GET_CHECK_STRING(abc, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence(abc))
    // push return value
    BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AbcToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Abc)", 10);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.BeatTracker class
//
#define BIND_CLASS_NAME "BeatTracker"
#define BIND_METHOD_NAME "constructor"
SQInteger BeatTrackerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BeatTracker *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "connection" as Midi.Input
    BIND_GET_CHECK_OBJECT(connection, 1, Midi, Input, midi, MidiInputPort)
    // get parameter 2 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = BeatTrackerFactory::instance().newBeatTracker(*connection, bpm))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger BeatTrackerNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.BeatTracker");
}
//
// Midi.BeatTracker start
//
#define BIND_METHOD_NAME "start"
SQInteger BeatTrackerstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.BeatTracker stop
//
#define BIND_METHOD_NAME "stop"
SQInteger BeatTrackerstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger BeatTrackerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.BeatTracker)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.ClockGenerator class
//
#define BIND_CLASS_NAME "ClockGenerator"
#define BIND_METHOD_NAME "constructor"
SQInteger ClockGeneratorCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    ClockGenerator *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = midi::ClockGeneratorFactory::instance().newMidiClockGenerator())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger ClockGeneratorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.ClockGenerator");
}
//
// Midi.ClockGenerator property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger ClockGeneratorGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ClockGenerator)
    // dispatch appropriate getter
    if(!strcmp(key, "midiout")) {
        HSQOBJECT ret = obj->midiout();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.ClockGenerator property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger ClockGeneratorSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ClockGenerator)
    // dispatch appropriate setter
    if(!strcmp(key, "midiout")) {
        return sq_throwerror(vm, "property \"midiout\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger ClockGeneratorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.ClockGenerator)", 21);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Connection class
//
#define BIND_CLASS_NAME "Connection"
SQInteger ConnectionCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.Connection");
}

SQInteger ConnectionNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Connection");
}
//
// Midi.Connection onControl
//
#define BIND_METHOD_NAME "onControl"
SQInteger ConnectiononControl(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onControl(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onMessage
//
#define BIND_METHOD_NAME "onMessage"
SQInteger ConnectiononMessage(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onMessage(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onNoteOff
//
#define BIND_METHOD_NAME "onNoteOff"
SQInteger ConnectiononNoteOff(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOff(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onNoteOn
//
#define BIND_METHOD_NAME "onNoteOn"
SQInteger ConnectiononNoteOn(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOn(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onProgramChange
//
#define BIND_METHOD_NAME "onProgramChange"
SQInteger ConnectiononProgramChange(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onProgramChange(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onSysex
//
#define BIND_METHOD_NAME "onSysex"
SQInteger ConnectiononSysex(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onSysex(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger ConnectionToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Connection)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Connector class
//
#define BIND_CLASS_NAME "Connector"
SQInteger ConnectorCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.Connector");
}

SQInteger ConnectorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Connector");
}
//
// Midi.Connector clear
//
#define BIND_METHOD_NAME "clear"
SQInteger Connectorclear(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->clear())
    } else {
        // get parameter 1 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 1, Time, Clock, time, Clock)
        // call the implementation
        BIND_WRAP_TRY(obj->clear(*clock))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connector schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Connectorschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &NoteObject) {
        // get parameter 1 "buffer" as Midi.Note
        BIND_GET_OBJECT(buffer, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &PatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure, *clock))
        }
    }
    else if(midi::Message *message = midi::binding::getMessage(arg1type, &arg1)) {
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure, *clock))
        }
    }
    else if(arg1type == &SequenceObject) {
        // get parameter 1 "message" as Midi.Sequence
        BIND_GET_OBJECT(message, 1, Midi, Sequence, midi, Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger ConnectorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Connector)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Control class
//
#define BIND_CLASS_NAME "Control"
#define BIND_METHOD_NAME "constructor"
SQInteger ControlCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Control *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "control" as integer
    BIND_GET_CHECK_INTEGER(control, 1)
    // get parameter 2 "value" as integer
    BIND_GET_CHECK_INTEGER(value, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = new Control(control, value))
    } else {
        // get parameter 3 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = new Control(control, value, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Control>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Control property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger ControlGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Control)
    // dispatch appropriate getter
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "controller")) {
        SQInteger ret = obj->controller();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "value")) {
        SQInteger ret = obj->value();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Control property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger ControlSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Control)
    // dispatch appropriate setter
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    if(!strcmp(key, "controller")) {
        return sq_throwerror(vm, "property \"controller\" is read-only");
    }
    if(!strcmp(key, "value")) {
        return sq_throwerror(vm, "property \"value\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger ControlToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Control)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Copyright class
//
#define BIND_CLASS_NAME "Copyright"
#define BIND_METHOD_NAME "constructor"
SQInteger CopyrightCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Copyright *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Copyright(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Copyright>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Copyright property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger CopyrightGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Copyright)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Copyright property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger CopyrightSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Copyright)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger CopyrightToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Copyright)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.CuePoint class
//
#define BIND_CLASS_NAME "CuePoint"
#define BIND_METHOD_NAME "constructor"
SQInteger CuePointCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    CuePoint *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new CuePoint(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<CuePoint>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.CuePoint property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger CuePointGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::CuePoint)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.CuePoint property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger CuePointSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::CuePoint)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger CuePointToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.CuePoint)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.File class
//
#define BIND_CLASS_NAME "File"
#define BIND_METHOD_NAME "constructor"
SQInteger FileCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    File *obj;
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj = new File())
    } else {
        HSQOBJECT arg1;
        sq_getstackobj(vm, 2, &arg1);
        if(arg1._type == OT_STRING) {
            BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
            // get parameter 1 "path" as string
            BIND_GET_STRING(path, 1)
            // call the implementation
            BIND_WRAP_TRY(obj = new File(path))
        }
        else if(arg1._type == OT_INSTANCE) {
        SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
        if(arg1type == &io::binding::FileObject) {
            // get parameter 1 "file" as IO.File
            BIND_GET_OBJECT(file, 1, IO, File, io, File)
            // call the implementation
            BIND_WRAP_TRY(obj = new File(*file))
        }
        else {
            return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
        }
        }
        else {
            return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<File>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.File property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger FileGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::File)
    // dispatch appropriate getter
    if(!strcmp(key, "ppq")) {
        SQInteger ret = obj->ppq();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "track")) {
        HSQOBJECT ret = obj->track();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "type")) {
        SQInteger ret = obj->type();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.File property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger FileSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::File)
    // dispatch appropriate setter
    if(!strcmp(key, "ppq")) {
        // get parameter 2 "ppq" as integer
        BIND_GET_CHECK_INTEGER(ppq, 2)
        BIND_WRAP_TRY(obj->ppq(ppq))
        return 0;
    }
    if(!strcmp(key, "track")) {
        // get parameter 2 "track" as array
        BIND_GET_CHECK_ARRAY(track, 2)
        BIND_WRAP_TRY(obj->track(track))
        return 0;
    }
    if(!strcmp(key, "type")) {
        // get parameter 2 "type" as integer
        BIND_GET_CHECK_INTEGER(type, 2)
        BIND_WRAP_TRY(obj->type(type))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.File save
//
#define BIND_METHOD_NAME "save"
SQInteger Filesave(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "path" as string
        BIND_GET_STRING(path, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->save(path))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &io::binding::FileObject) {
        // get parameter 1 "file" as IO.File
        BIND_GET_OBJECT(file, 1, IO, File, io, File)
        // call the implementation
        BIND_WRAP_TRY(obj->save(*file))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.File tracks
//
#define BIND_METHOD_NAME "tracks"
SQInteger Filetracks(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->tracks())
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger FileToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.File)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Input class
//
#define BIND_CLASS_NAME "Input"
#define BIND_METHOD_NAME "constructor"
SQInteger InputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MidiInputPort *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = MidiInputPortFactory::instance().newMidiInputPort(name))
    } else {
        HSQOBJECT arg2;
        sq_getstackobj(vm, 3, &arg2);
        if(arg2._type == OT_STRING) {
            // get parameter 2 "connection" as string
            BIND_GET_STRING(connection, 2)
            // call the implementation
            BIND_WRAP_TRY(obj = MidiInputPortFactory::instance().newMidiInputPort(name, connection))
        }
        else if(arg2._type == OT_INSTANCE) {
        SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
        if(arg2type == &bipscript::binding::regexObject) {
            BIND_NARGS_CHECK_MIN(2) // check min parameter count
            // get parameter 2 "connection" as regex
            BIND_GET_OBJECT(connection, 2, , regex, bipscript, Regex)
            // call the implementation
            BIND_WRAP_TRY(obj = MidiInputPortFactory::instance().newMidiInputPort(name, *connection))
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
        }
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger InputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Input");
}
//
// Midi.Input property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger InputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiInputPort)
    // dispatch appropriate getter
    if(!strcmp(key, "midiout")) {
        HSQOBJECT ret = obj->midiout();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Input property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger InputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiInputPort)
    // dispatch appropriate setter
    if(!strcmp(key, "midiout")) {
        return sq_throwerror(vm, "property \"midiout\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Input connect
//
#define BIND_METHOD_NAME "connect"
SQInteger Inputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "connection" as string
        BIND_GET_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(connection))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &bipscript::binding::regexObject) {
        // get parameter 1 "connection" as regex
        BIND_GET_OBJECT(connection, 1, , regex, bipscript, Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(*connection))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input disconnect
//
#define BIND_METHOD_NAME "disconnect"
SQInteger Inputdisconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect())
    } else {
        // get parameter 1 "connection" as string
        BIND_GET_CHECK_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect(connection))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onControl
//
#define BIND_METHOD_NAME "onControl"
SQInteger InputonControl(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onControl(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onMessage
//
#define BIND_METHOD_NAME "onMessage"
SQInteger InputonMessage(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onMessage(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onNoteOff
//
#define BIND_METHOD_NAME "onNoteOff"
SQInteger InputonNoteOff(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOff(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onNoteOn
//
#define BIND_METHOD_NAME "onNoteOn"
SQInteger InputonNoteOn(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOn(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onProgramChange
//
#define BIND_METHOD_NAME "onProgramChange"
SQInteger InputonProgramChange(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onProgramChange(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onSysex
//
#define BIND_METHOD_NAME "onSysex"
SQInteger InputonSysex(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onSysex(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger InputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Input)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Instrument class
//
#define BIND_CLASS_NAME "Instrument"
#define BIND_METHOD_NAME "constructor"
SQInteger InstrumentCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Instrument *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Instrument(name))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Instrument>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Instrument property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger InstrumentGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Instrument)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Instrument property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger InstrumentSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Instrument)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger InstrumentToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Instrument)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.KeySignature class
//
#define BIND_CLASS_NAME "KeySignature"
#define BIND_METHOD_NAME "constructor"
SQInteger KeySignatureCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    KeySignature *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "count" as integer
    BIND_GET_CHECK_INTEGER(count, 1)
    // get parameter 2 "minor" as bool
    BIND_GET_CHECK_BOOL(minor, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = new KeySignature(count, minor))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<KeySignature>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.KeySignature property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger KeySignatureGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::KeySignature)
    // dispatch appropriate getter
    if(!strcmp(key, "count")) {
        SQInteger ret = obj->count();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "minor")) {
        SQBool ret = obj->minor();
        BIND_PUSH_BOOL(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.KeySignature property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger KeySignatureSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::KeySignature)
    // dispatch appropriate setter
    if(!strcmp(key, "count")) {
        return sq_throwerror(vm, "property \"count\" is read-only");
    }
    if(!strcmp(key, "minor")) {
        return sq_throwerror(vm, "property \"minor\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger KeySignatureToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.KeySignature)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Lyric class
//
#define BIND_CLASS_NAME "Lyric"
#define BIND_METHOD_NAME "constructor"
SQInteger LyricCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Lyric *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Lyric(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Lyric>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Lyric property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger LyricGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Lyric)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Lyric property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger LyricSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Lyric)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger LyricToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Lyric)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.MML class
//
#define BIND_CLASS_NAME "MML"
#define BIND_METHOD_NAME "constructor"
SQInteger MMLCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MML *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new MML())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<MML>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.MML sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger MMLsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MML)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "mml" as string
    BIND_GET_CHECK_STRING(mml, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence(mml))
    // push return value
    BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MMLToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.MML)", 10);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Marker class
//
#define BIND_CLASS_NAME "Marker"
#define BIND_METHOD_NAME "constructor"
SQInteger MarkerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Marker *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Marker(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Marker>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Marker property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MarkerGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Marker)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Marker property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MarkerSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Marker)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MarkerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Marker)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.MessageEvent class
//
#define BIND_CLASS_NAME "MessageEvent"
SQInteger MessageEventCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.MessageEvent");
}

SQInteger MessageEventNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.MessageEvent");
}
//
// Midi.MessageEvent property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MessageEventGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MessageEvent)
    // dispatch appropriate getter
    if(!strcmp(key, "measure")) {
        SQFloat ret = obj->measure();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    if(!strcmp(key, "message")) {
        midi::Message* ret = obj->message();
        midi::binding::pushMessage(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.MessageEvent property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MessageEventSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MessageEvent)
    // dispatch appropriate setter
    if(!strcmp(key, "measure")) {
        return sq_throwerror(vm, "property \"measure\" is read-only");
    }
    if(!strcmp(key, "message")) {
        return sq_throwerror(vm, "property \"message\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MessageEventToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.MessageEvent)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Note class
//
#define BIND_CLASS_NAME "Note"
#define BIND_METHOD_NAME "constructor"
SQInteger NoteCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Note *obj;
    BIND_NARGS_CHECK_MIN(3) // check min parameter count
    BIND_NARGS_CHECK_MAX(4) // check max parameter count
    // get parameter 1 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 1)
    // get parameter 2 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 2)
    // get parameter 3 "duration" as float
    BIND_GET_CHECK_FLOAT(duration, 3)
    if(numargs == 4) {
        // call the implementation
        BIND_WRAP_TRY(obj = new Note(pitch, velocity, duration))
    } else {
        // get parameter 4 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 4)
        // call the implementation
        BIND_WRAP_TRY(obj = new Note(pitch, velocity, duration, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Note>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Note property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger NoteGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Note)
    // dispatch appropriate getter
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "duration")) {
        SQFloat ret = obj->duration();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    if(!strcmp(key, "pitch")) {
        SQInteger ret = obj->pitch();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "velocity")) {
        SQInteger ret = obj->velocity();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Note property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger NoteSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Note)
    // dispatch appropriate setter
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    if(!strcmp(key, "duration")) {
        return sq_throwerror(vm, "property \"duration\" is read-only");
    }
    if(!strcmp(key, "pitch")) {
        return sq_throwerror(vm, "property \"pitch\" is read-only");
    }
    if(!strcmp(key, "velocity")) {
        return sq_throwerror(vm, "property \"velocity\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger NoteToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Note)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.NoteEvent class
//
#define BIND_CLASS_NAME "NoteEvent"
SQInteger NoteEventCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.NoteEvent");
}

SQInteger NoteEventNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.NoteEvent");
}
//
// Midi.NoteEvent property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger NoteEventGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteEvent)
    // dispatch appropriate getter
    if(!strcmp(key, "measure")) {
        SQFloat ret = obj->measure();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    if(!strcmp(key, "note")) {
        midi::Note* ret = obj->note();
        BIND_PUSH_OBJECT(ret->addref(), Note, midi, Note, Decrement)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.NoteEvent property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger NoteEventSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteEvent)
    // dispatch appropriate setter
    if(!strcmp(key, "measure")) {
        return sq_throwerror(vm, "property \"measure\" is read-only");
    }
    if(!strcmp(key, "note")) {
        return sq_throwerror(vm, "property \"note\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger NoteEventToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.NoteEvent)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.NoteOff class
//
#define BIND_CLASS_NAME "NoteOff"
#define BIND_METHOD_NAME "constructor"
SQInteger NoteOffCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    NoteOff *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = new NoteOff(pitch))
    } else {
        // get parameter 2 "velocity" as integer
        BIND_GET_CHECK_INTEGER(velocity, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj = new NoteOff(pitch, velocity))
        } else {
            // get parameter 3 "channel" as integer
            BIND_GET_CHECK_INTEGER(channel, 3)
            // call the implementation
            BIND_WRAP_TRY(obj = new NoteOff(pitch, velocity, channel))
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<NoteOff>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOff property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger NoteOffGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOff)
    // dispatch appropriate getter
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "pitch")) {
        SQInteger ret = obj->pitch();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "velocity")) {
        SQInteger ret = obj->velocity();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOff property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger NoteOffSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOff)
    // dispatch appropriate setter
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    if(!strcmp(key, "pitch")) {
        return sq_throwerror(vm, "property \"pitch\" is read-only");
    }
    if(!strcmp(key, "velocity")) {
        return sq_throwerror(vm, "property \"velocity\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger NoteOffToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.NoteOff)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.NoteOn class
//
#define BIND_CLASS_NAME "NoteOn"
#define BIND_METHOD_NAME "constructor"
SQInteger NoteOnCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    NoteOn *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 1)
    // get parameter 2 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = new NoteOn(pitch, velocity))
    } else {
        // get parameter 3 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = new NoteOn(pitch, velocity, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<NoteOn>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOn property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger NoteOnGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOn)
    // dispatch appropriate getter
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "pitch")) {
        SQInteger ret = obj->pitch();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "velocity")) {
        SQInteger ret = obj->velocity();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOn property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger NoteOnSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOn)
    // dispatch appropriate setter
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    if(!strcmp(key, "pitch")) {
        return sq_throwerror(vm, "property \"pitch\" is read-only");
    }
    if(!strcmp(key, "velocity")) {
        return sq_throwerror(vm, "property \"velocity\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger NoteOnToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.NoteOn)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Output class
//
#define BIND_CLASS_NAME "Output"
#define BIND_METHOD_NAME "constructor"
SQInteger OutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MidiOutputPort *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = MidiOutputPortFactory::instance().newMidiOutputPort(name))
    } else {
        // get parameter 2 "connection" as string
        BIND_GET_CHECK_STRING(connection, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = MidiOutputPortFactory::instance().newMidiOutputPort(name, connection))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger OutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Output");
}
//
// Midi.Output property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger OutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiOutputPort)
    // dispatch appropriate getter
    if(!strcmp(key, "midiin")) {
        HSQOBJECT ret = obj->midiin();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Output property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger OutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiOutputPort)
    // dispatch appropriate setter
    if(!strcmp(key, "midiin")) {
        return sq_throwerror(vm, "property \"midiin\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Output clear
//
#define BIND_METHOD_NAME "clear"
SQInteger Outputclear(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->clear())
    } else {
        // get parameter 1 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 1, Time, Clock, time, Clock)
        // call the implementation
        BIND_WRAP_TRY(obj->clear(*clock))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Output connect
//
#define BIND_METHOD_NAME "connect"
SQInteger Outputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "connection" as string
        BIND_GET_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(connection))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &bipscript::binding::regexObject) {
        // get parameter 1 "connection" as regex
        BIND_GET_OBJECT(connection, 1, , regex, bipscript, Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(*connection))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Output disconnect
//
#define BIND_METHOD_NAME "disconnect"
SQInteger Outputdisconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect())
    } else {
        // get parameter 1 "connection" as string
        BIND_GET_CHECK_STRING(connection, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->disconnect(connection))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Output remove
//
#define BIND_METHOD_NAME "remove"
SQInteger Outputremove(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MessageEventObject) {
        // get parameter 1 "event" as Midi.MessageEvent
        BIND_GET_OBJECT(event, 1, Midi, MessageEvent, midi, MessageEvent)
        // call the implementation
        BIND_WRAP_TRY(obj->remove(*event))
    }
    else if(arg1type == &NoteEventObject) {
        // get parameter 1 "event" as Midi.NoteEvent
        BIND_GET_OBJECT(event, 1, Midi, NoteEvent, midi, NoteEvent)
        // call the implementation
        BIND_WRAP_TRY(obj->remove(*event))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.MessageEvent, Midi.NoteEvent}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.MessageEvent, Midi.NoteEvent}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Output schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger Outputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &NoteObject) {
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::NoteEvent* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*note, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*note, measure, *clock))
        }
        // push return value
        BIND_PUSH_OBJECT(ret->addref(), NoteEvent, midi, NoteEvent, Decrement)
        return 1;

    }
    else if(arg1type == &PatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else if(midi::Message *message = midi::binding::getMessage(arg1type, &arg1)) {
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::MessageEvent* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*message, measure, *clock))
        }
        // push return value
        BIND_PUSH_OBJECT(ret->addref(), MessageEvent, midi, MessageEvent, Decrement)
        return 1;

    }
    else if(arg1type == &SequenceObject) {
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi, Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::Sequence* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*sequence, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*sequence, measure, *clock))
        }
        // push return value
        BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
        return 1;

    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }}
#undef BIND_METHOD_NAME

//
// Midi.Output send
//
#define BIND_METHOD_NAME "send"
SQInteger Outputsend(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(midi::Message *message = midi::binding::getMessage(arg1type, &arg1)) {
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*message))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            if(numargs == 3) {
                // call the implementation
                BIND_WRAP_TRY(obj->send(*message, measure))
            } else {
                // get parameter 3 "clock" as Time.Clock
                BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
                // call the implementation
                BIND_WRAP_TRY(obj->send(*message, measure, *clock))
            }
        }
    }
    else if(arg1type == &NoteObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*note, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*note, measure, *clock))
        }
    }
    else if(arg1type == &PatternObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*pattern, measure, *clock))
        }
    }
    else if(arg1type == &SequenceObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi, Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*sequence, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time, Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*sequence, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger OutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Output)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Pattern class
//
#define BIND_CLASS_NAME "Pattern"
#define BIND_METHOD_NAME "constructor"
SQInteger PatternCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Pattern *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Pattern())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Pattern>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger PatternGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        BIND_GET_THIS_PROPERTY(midi::Pattern)
        midi::NoteEvent* ret = (*obj)[index];
        if(ret) {
            BIND_PUSH_OBJECT(ret->addref(), NoteEvent, midi, NoteEvent, Decrement)
            return 1;
        }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME
//
// Midi.Pattern _nexti
//
SQInteger PatternNexti(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
	BIND_GET_THIS_PROPERTY(Pattern)
    if (argtype == OT_NULL) {
        if(obj->len()) {
            sq_pushinteger(vm, 0);
        }
		return 1;
    } else if (argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        if ((size_t)index + 1 < obj->len()) {
            sq_pushinteger(vm, index + 1);
            return 1;
        }
    }
    sq_pushnull(vm);
	return 1;
}
//
// Midi.Pattern add
//
#define BIND_METHOD_NAME "add"
SQInteger Patternadd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &NoteObject) {
        BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->add(*note, measure))
    }
    else if(arg1type == &PatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern, measure))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern filter
//
#define BIND_METHOD_NAME "filter"
SQInteger Patternfilter(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "filter" as function
    BIND_GET_CHECK_FUNCTION(filter, 1)
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->filter(filter))
    // push return value
    BIND_PUSH_OBJECT(ret, Pattern, midi, Pattern, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern len
//
#define BIND_METHOD_NAME "len"
SQInteger Patternlen(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->len())
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern print
//
#define BIND_METHOD_NAME "print"
SQInteger Patternprint(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->print())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger Patternsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence())
    // push return value
    BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern slice
//
#define BIND_METHOD_NAME "slice"
SQInteger Patternslice(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "start" as float
    BIND_GET_CHECK_FLOAT(start, 1)
    // get parameter 2 "end" as float
    BIND_GET_CHECK_FLOAT(end, 2)
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->slice(start, end))
    // push return value
    BIND_PUSH_OBJECT(ret, Pattern, midi, Pattern, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern transpose
//
#define BIND_METHOD_NAME "transpose"
SQInteger Patterntranspose(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "amount" as integer
    BIND_GET_CHECK_INTEGER(amount, 1)
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->transpose(amount))
    // push return value
    BIND_PUSH_OBJECT(ret, Pattern, midi, Pattern, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger PatternToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Pattern)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.PitchBend class
//
#define BIND_CLASS_NAME "PitchBend"
#define BIND_METHOD_NAME "constructor"
SQInteger PitchBendCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    PitchBend *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "amount" as integer
    BIND_GET_CHECK_INTEGER(amount, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = new PitchBend(amount))
    } else {
        // get parameter 2 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = new PitchBend(amount, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<PitchBend>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.PitchBend property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger PitchBendGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::PitchBend)
    // dispatch appropriate getter
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "value")) {
        SQInteger ret = obj->value();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.PitchBend property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger PitchBendSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::PitchBend)
    // dispatch appropriate setter
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    if(!strcmp(key, "value")) {
        return sq_throwerror(vm, "property \"value\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger PitchBendToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.PitchBend)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.ProgramChange class
//
#define BIND_CLASS_NAME "ProgramChange"
#define BIND_METHOD_NAME "constructor"
SQInteger ProgramChangeCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    ProgramChange *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "program" as integer
    BIND_GET_CHECK_INTEGER(program, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = new ProgramChange(program))
    } else {
        // get parameter 2 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = new ProgramChange(program, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<ProgramChange>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.ProgramChange property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger ProgramChangeGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ProgramChange)
    // dispatch appropriate getter
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "program")) {
        SQInteger ret = obj->program();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.ProgramChange property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger ProgramChangeSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ProgramChange)
    // dispatch appropriate setter
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    if(!strcmp(key, "program")) {
        return sq_throwerror(vm, "property \"program\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger ProgramChangeToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.ProgramChange)", 20);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Sequence class
//
#define BIND_CLASS_NAME "Sequence"
#define BIND_METHOD_NAME "constructor"
SQInteger SequenceCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Sequence *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Sequence())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger SequenceGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        BIND_GET_THIS_PROPERTY(midi::Sequence)
        midi::MessageEvent* ret = (*obj)[index];
        if(ret) {
            BIND_PUSH_OBJECT(ret->addref(), MessageEvent, midi, MessageEvent, Decrement)
            return 1;
        }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME
//
// Midi.Sequence _nexti
//
SQInteger SequenceNexti(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
	BIND_GET_THIS_PROPERTY(Sequence)
    if (argtype == OT_NULL) {
        if(obj->len()) {
            sq_pushinteger(vm, 0);
        }
		return 1;
    } else if (argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        if ((size_t)index + 1 < obj->len()) {
            sq_pushinteger(vm, index + 1);
            return 1;
        }
    }
    sq_pushnull(vm);
	return 1;
}
//
// Midi.Sequence add
//
#define BIND_METHOD_NAME "add"
SQInteger Sequenceadd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(midi::Message *message = midi::binding::getMessage(arg1type, &arg1)) {
        BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->add(*message, measure))
    }
    else if(arg1type == &NoteObject) {
        BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
        // get parameter 1 "message" as Midi.Note
        BIND_GET_OBJECT(message, 1, Midi, Note, midi, Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->add(*message, measure))
    }
    else if(arg1type == &PatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern, measure))
        }
    }
    else if(arg1type == &SequenceObject) {
        // get parameter 1 "pattern" as Midi.Sequence
        BIND_GET_OBJECT(pattern, 1, Midi, Sequence, midi, Sequence)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern, measure))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence clear
//
#define BIND_METHOD_NAME "clear"
SQInteger Sequenceclear(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->clear())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence filter
//
#define BIND_METHOD_NAME "filter"
SQInteger Sequencefilter(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "filter" as function
    BIND_GET_CHECK_FUNCTION(filter, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->filter(filter))
    // push return value
    BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence len
//
#define BIND_METHOD_NAME "len"
SQInteger Sequencelen(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->len())
    // push return value
    BIND_PUSH_INTEGER(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence pattern
//
#define BIND_METHOD_NAME "pattern"
SQInteger Sequencepattern(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->pattern())
    // push return value
    BIND_PUSH_OBJECT(ret, Pattern, midi, Pattern, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger SequenceToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Sequence)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Sysex class
//
#define BIND_CLASS_NAME "Sysex"
#define BIND_METHOD_NAME "constructor"
SQInteger SysexCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Sysex *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "buffer" as buffer
    BIND_GET_CHECK_OBJECT(buffer, 1, , buffer, bipscript, Buffer)
    // call the implementation
    BIND_WRAP_TRY(obj = new Sysex(*buffer))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Sysex>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sysex property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger SysexGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Sysex)
    // dispatch appropriate getter
    if(!strcmp(key, "data")) {
        bipscript::Buffer* ret = obj->data();
        BIND_PUSH_OBJECT(ret->addref(), buffer, bipscript, Buffer, Decrement)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Sysex property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger SysexSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Sysex)
    // dispatch appropriate setter
    if(!strcmp(key, "data")) {
        return sq_throwerror(vm, "property \"data\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger SysexToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Sysex)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Tab class
//
#define BIND_CLASS_NAME "Tab"
#define BIND_METHOD_NAME "constructor"
SQInteger TabCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Tab *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Tab())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Tab>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TabGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Tab)
    // dispatch appropriate getter
    if(!strcmp(key, "division")) {
        SQInteger ret = obj->division();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "linewrap")) {
        SQInteger ret = obj->linewrap();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Tab property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TabSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Tab)
    // dispatch appropriate setter
    if(!strcmp(key, "division")) {
        // get parameter 2 "division" as integer
        BIND_GET_CHECK_INTEGER(division, 2)
        BIND_WRAP_TRY(obj->division(division))
        return 0;
    }
    if(!strcmp(key, "linewrap")) {
        // get parameter 2 "linewrap" as integer
        BIND_GET_CHECK_INTEGER(linewrap, 2)
        BIND_WRAP_TRY(obj->linewrap(linewrap))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Tab define
//
#define BIND_METHOD_NAME "define"
SQInteger Tabdefine(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(4) // check exact parameter count
    // get parameter 1 "prefix" as string
    BIND_GET_CHECK_STRING(prefix, 1)
    // get parameter 2 "strike" as string
    BIND_GET_CHECK_STRING(strike, 2)
    // get parameter 3 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 3)
    // get parameter 4 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 4)
    // call the implementation
    BIND_WRAP_TRY(obj->define(prefix, strike, pitch, velocity))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger Tabsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "tab" as string
    BIND_GET_CHECK_STRING(tab, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence(tab))
    // push return value
    BIND_PUSH_OBJECT(ret, Sequence, midi, Sequence, Delete)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab string
//
#define BIND_METHOD_NAME "string"
SQInteger Tabstring(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // return value
    std::string ret;
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &PatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi, Pattern)
        // call the implementation
        BIND_WRAP_TRY(ret = obj->string(*pattern))
    }
    else if(arg1type == &SequenceObject) {
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi, Sequence)
        // call the implementation
        BIND_WRAP_TRY(ret = obj->string(*sequence))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Pattern, Midi.Sequence}");
    }
    // push return value
    BIND_PUSH_STRING(ret)
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab velocity
//
#define BIND_METHOD_NAME "velocity"
SQInteger Tabvelocity(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "strike" as string
    BIND_GET_CHECK_STRING(strike, 1)
    // get parameter 2 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 2)
    // call the implementation
    BIND_WRAP_TRY(obj->velocity(strike, velocity))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger TabToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Tab)", 10);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.TempoChange class
//
#define BIND_CLASS_NAME "TempoChange"
#define BIND_METHOD_NAME "constructor"
SQInteger TempoChangeCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    TempoChange *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "tempo" as float
    BIND_GET_CHECK_FLOAT(tempo, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new TempoChange(tempo))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<TempoChange>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.TempoChange property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TempoChangeGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TempoChange)
    // dispatch appropriate getter
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.TempoChange property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TempoChangeSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TempoChange)
    // dispatch appropriate setter
    if(!strcmp(key, "tempo")) {
        return sq_throwerror(vm, "property \"tempo\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger TempoChangeToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.TempoChange)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Text class
//
#define BIND_CLASS_NAME "Text"
#define BIND_METHOD_NAME "constructor"
SQInteger TextCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Text *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Text(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Text>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Text property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TextGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Text)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Text property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TextSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Text)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger TextToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Text)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.TimeSignature class
//
#define BIND_CLASS_NAME "TimeSignature"
#define BIND_METHOD_NAME "constructor"
SQInteger TimeSignatureCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    TimeSignature *obj;
    BIND_NARGS_CHECK_EXACT(4) // check exact parameter count
    // get parameter 1 "numerator" as integer
    BIND_GET_CHECK_INTEGER(numerator, 1)
    // get parameter 2 "denominator" as integer
    BIND_GET_CHECK_INTEGER(denominator, 2)
    // get parameter 3 "ppq" as integer
    BIND_GET_CHECK_INTEGER(ppq, 3)
    // get parameter 4 "dspq" as integer
    BIND_GET_CHECK_INTEGER(dspq, 4)
    // call the implementation
    BIND_WRAP_TRY(obj = new TimeSignature(numerator, denominator, ppq, dspq))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<TimeSignature>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.TimeSignature property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TimeSignatureGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TimeSignature)
    // dispatch appropriate getter
    if(!strcmp(key, "denominator")) {
        SQInteger ret = obj->denominator();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    if(!strcmp(key, "numerator")) {
        SQInteger ret = obj->numerator();
        BIND_PUSH_INTEGER(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.TimeSignature property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TimeSignatureSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TimeSignature)
    // dispatch appropriate setter
    if(!strcmp(key, "denominator")) {
        return sq_throwerror(vm, "property \"denominator\" is read-only");
    }
    if(!strcmp(key, "numerator")) {
        return sq_throwerror(vm, "property \"numerator\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger TimeSignatureToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.TimeSignature)", 20);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.TrackName class
//
#define BIND_CLASS_NAME "TrackName"
#define BIND_METHOD_NAME "constructor"
SQInteger TrackNameCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    TrackName *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new TrackName(name))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<TrackName>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.TrackName property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger TrackNameGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TrackName)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret)
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.TrackName property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger TrackNameSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TrackName)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger TrackNameToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.TrackName)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

SQInteger MidiNewSlot(HSQUIRRELVM vm)
{
	return sq_throwerror(vm, "package table does not support the new slot operator");
}

void bindMidi(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Midi", 4);

    // static methods
    binding.addMethod("abc", 3, &Midiabc);

    // create class Midi.Abc
    binding.startClass("Abc", 3, AbcObject);
    binding.addMethod("constructor", 11, &AbcCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Abc>); // clone
    binding.addMethod("abc", 3, &Abcabc);
    binding.addMethod("file", 4, &Abcfile);
    binding.addMethod("sequence", 8, &Abcsequence);
    binding.addMethod("_tostring", 9, &AbcToString);
    binding.endClass();

    // create class Midi.BeatTracker
    binding.startClass("BeatTracker", 11, BeatTrackerObject);
    binding.addMethod("constructor", 11, &BeatTrackerCtor); // ctor
    binding.addMethod("_cloned", 7, &BeatTrackerNoclone); // unclonable
    binding.addMethod("start", 5, &BeatTrackerstart);
    binding.addMethod("stop", 4, &BeatTrackerstop);
    binding.addMethod("_tostring", 9, &BeatTrackerToString);
    binding.endClass();

    // create class Midi.ClockGenerator
    binding.startClass("ClockGenerator", 14, ClockGeneratorObject);
    binding.addMethod("constructor", 11, &ClockGeneratorCtor); // ctor
    binding.addMethod("_cloned", 7, &ClockGeneratorNoclone); // unclonable
    binding.addMethod("_get", 4, &ClockGeneratorGetters);
    binding.addMethod("_set", 4, &ClockGeneratorSetters);
    binding.addMethod("_tostring", 9, &ClockGeneratorToString);
    binding.endClass();

    // create class Midi.Connection
    binding.startClass("Connection", 10, ConnectionObject);
    binding.addMethod("constructor", 11, &ConnectionCtor); // ctor
    binding.addMethod("_cloned", 7, &ConnectionNoclone); // unclonable
    binding.addMethod("onControl", 9, &ConnectiononControl);
    binding.addMethod("onMessage", 9, &ConnectiononMessage);
    binding.addMethod("onNoteOff", 9, &ConnectiononNoteOff);
    binding.addMethod("onNoteOn", 8, &ConnectiononNoteOn);
    binding.addMethod("onProgramChange", 15, &ConnectiononProgramChange);
    binding.addMethod("onSysex", 7, &ConnectiononSysex);
    binding.addMethod("_tostring", 9, &ConnectionToString);
    binding.endClass();

    // create class Midi.Connector
    binding.startClass("Connector", 9, ConnectorObject);
    binding.addMethod("constructor", 11, &ConnectorCtor); // ctor
    binding.addMethod("_cloned", 7, &ConnectorNoclone); // unclonable
    binding.addMethod("clear", 5, &Connectorclear);
    binding.addMethod("schedule", 8, &Connectorschedule);
    binding.addMethod("_tostring", 9, &ConnectorToString);
    binding.endClass();

    // create class Midi.Control
    binding.startClass("Control", 7, ControlObject);
    binding.addMethod("constructor", 11, &ControlCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Control>); // clone
    binding.addMethod("_get", 4, &ControlGetters);
    binding.addMethod("_set", 4, &ControlSetters);
    binding.addMethod("_tostring", 9, &ControlToString);
    binding.endClass();

    // create class Midi.Copyright
    binding.startClass("Copyright", 9, CopyrightObject);
    binding.addMethod("constructor", 11, &CopyrightCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Copyright>); // clone
    binding.addMethod("_get", 4, &CopyrightGetters);
    binding.addMethod("_set", 4, &CopyrightSetters);
    binding.addMethod("_tostring", 9, &CopyrightToString);
    binding.endClass();

    // create class Midi.CuePoint
    binding.startClass("CuePoint", 8, CuePointObject);
    binding.addMethod("constructor", 11, &CuePointCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<CuePoint>); // clone
    binding.addMethod("_get", 4, &CuePointGetters);
    binding.addMethod("_set", 4, &CuePointSetters);
    binding.addMethod("_tostring", 9, &CuePointToString);
    binding.endClass();

    // create class Midi.File
    binding.startClass("File", 4, FileObject);
    binding.addMethod("constructor", 11, &FileCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<File>); // clone
    binding.addMethod("_get", 4, &FileGetters);
    binding.addMethod("_set", 4, &FileSetters);
    binding.addMethod("save", 4, &Filesave);
    binding.addMethod("tracks", 6, &Filetracks);
    binding.addMethod("_tostring", 9, &FileToString);
    binding.endClass();

    // create class Midi.Input
    binding.startClass("Input", 5, InputObject);
    binding.addMethod("constructor", 11, &InputCtor); // ctor
    binding.addMethod("_cloned", 7, &InputNoclone); // unclonable
    binding.addMethod("_get", 4, &InputGetters);
    binding.addMethod("_set", 4, &InputSetters);
    binding.addMethod("connect", 7, &Inputconnect);
    binding.addMethod("disconnect", 10, &Inputdisconnect);
    binding.addMethod("onControl", 9, &InputonControl);
    binding.addMethod("onMessage", 9, &InputonMessage);
    binding.addMethod("onNoteOff", 9, &InputonNoteOff);
    binding.addMethod("onNoteOn", 8, &InputonNoteOn);
    binding.addMethod("onProgramChange", 15, &InputonProgramChange);
    binding.addMethod("onSysex", 7, &InputonSysex);
    binding.addMethod("_tostring", 9, &InputToString);
    binding.endClass();

    // create class Midi.Instrument
    binding.startClass("Instrument", 10, InstrumentObject);
    binding.addMethod("constructor", 11, &InstrumentCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Instrument>); // clone
    binding.addMethod("_get", 4, &InstrumentGetters);
    binding.addMethod("_set", 4, &InstrumentSetters);
    binding.addMethod("_tostring", 9, &InstrumentToString);
    binding.endClass();

    // create class Midi.KeySignature
    binding.startClass("KeySignature", 12, KeySignatureObject);
    binding.addMethod("constructor", 11, &KeySignatureCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<KeySignature>); // clone
    binding.addMethod("_get", 4, &KeySignatureGetters);
    binding.addMethod("_set", 4, &KeySignatureSetters);
    binding.addMethod("_tostring", 9, &KeySignatureToString);
    binding.endClass();

    // create class Midi.Lyric
    binding.startClass("Lyric", 5, LyricObject);
    binding.addMethod("constructor", 11, &LyricCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Lyric>); // clone
    binding.addMethod("_get", 4, &LyricGetters);
    binding.addMethod("_set", 4, &LyricSetters);
    binding.addMethod("_tostring", 9, &LyricToString);
    binding.endClass();

    // create class Midi.MML
    binding.startClass("MML", 3, MMLObject);
    binding.addMethod("constructor", 11, &MMLCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<MML>); // clone
    binding.addMethod("sequence", 8, &MMLsequence);
    binding.addMethod("_tostring", 9, &MMLToString);
    binding.endClass();

    // create class Midi.Marker
    binding.startClass("Marker", 6, MarkerObject);
    binding.addMethod("constructor", 11, &MarkerCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Marker>); // clone
    binding.addMethod("_get", 4, &MarkerGetters);
    binding.addMethod("_set", 4, &MarkerSetters);
    binding.addMethod("_tostring", 9, &MarkerToString);
    binding.endClass();

    // create class Midi.MessageEvent
    binding.startClass("MessageEvent", 12, MessageEventObject);
    binding.addMethod("constructor", 11, &MessageEventCtor); // ctor
    binding.addMethod("_cloned", 7, &MessageEventNoclone); // unclonable
    binding.addMethod("_get", 4, &MessageEventGetters);
    binding.addMethod("_set", 4, &MessageEventSetters);
    binding.addMethod("_tostring", 9, &MessageEventToString);
    binding.endClass();

    // create class Midi.Note
    binding.startClass("Note", 4, NoteObject);
    binding.addMethod("constructor", 11, &NoteCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Note>); // clone
    binding.addMethod("_get", 4, &NoteGetters);
    binding.addMethod("_set", 4, &NoteSetters);
    binding.addMethod("_tostring", 9, &NoteToString);
    binding.endClass();

    // create class Midi.NoteEvent
    binding.startClass("NoteEvent", 9, NoteEventObject);
    binding.addMethod("constructor", 11, &NoteEventCtor); // ctor
    binding.addMethod("_cloned", 7, &NoteEventNoclone); // unclonable
    binding.addMethod("_get", 4, &NoteEventGetters);
    binding.addMethod("_set", 4, &NoteEventSetters);
    binding.addMethod("_tostring", 9, &NoteEventToString);
    binding.endClass();

    // create class Midi.NoteOff
    binding.startClass("NoteOff", 7, NoteOffObject);
    binding.addMethod("constructor", 11, &NoteOffCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<NoteOff>); // clone
    binding.addMethod("_get", 4, &NoteOffGetters);
    binding.addMethod("_set", 4, &NoteOffSetters);
    binding.addMethod("_tostring", 9, &NoteOffToString);
    binding.endClass();

    // create class Midi.NoteOn
    binding.startClass("NoteOn", 6, NoteOnObject);
    binding.addMethod("constructor", 11, &NoteOnCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<NoteOn>); // clone
    binding.addMethod("_get", 4, &NoteOnGetters);
    binding.addMethod("_set", 4, &NoteOnSetters);
    binding.addMethod("_tostring", 9, &NoteOnToString);
    binding.endClass();

    // create class Midi.Output
    binding.startClass("Output", 6, OutputObject);
    binding.addMethod("constructor", 11, &OutputCtor); // ctor
    binding.addMethod("_cloned", 7, &OutputNoclone); // unclonable
    binding.addMethod("_get", 4, &OutputGetters);
    binding.addMethod("_set", 4, &OutputSetters);
    binding.addMethod("clear", 5, &Outputclear);
    binding.addMethod("connect", 7, &Outputconnect);
    binding.addMethod("disconnect", 10, &Outputdisconnect);
    binding.addMethod("remove", 6, &Outputremove);
    binding.addMethod("schedule", 8, &Outputschedule);
    binding.addMethod("send", 4, &Outputsend);
    binding.addMethod("_tostring", 9, &OutputToString);
    binding.endClass();

    // create class Midi.Pattern
    binding.startClass("Pattern", 7, PatternObject);
    binding.addMethod("constructor", 11, &PatternCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Pattern>); // clone
    binding.addMethod("_get", 4, &PatternGetters);
    binding.addMethod("_nexti", 6, &PatternNexti);
    binding.addMethod("add", 3, &Patternadd);
    binding.addMethod("filter", 6, &Patternfilter);
    binding.addMethod("len", 3, &Patternlen);
    binding.addMethod("print", 5, &Patternprint);
    binding.addMethod("sequence", 8, &Patternsequence);
    binding.addMethod("slice", 5, &Patternslice);
    binding.addMethod("transpose", 9, &Patterntranspose);
    binding.addMethod("_tostring", 9, &PatternToString);
    binding.endClass();

    // create class Midi.PitchBend
    binding.startClass("PitchBend", 9, PitchBendObject);
    binding.addMethod("constructor", 11, &PitchBendCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<PitchBend>); // clone
    binding.addMethod("_get", 4, &PitchBendGetters);
    binding.addMethod("_set", 4, &PitchBendSetters);
    binding.addMethod("_tostring", 9, &PitchBendToString);
    binding.endClass();

    // create class Midi.ProgramChange
    binding.startClass("ProgramChange", 13, ProgramChangeObject);
    binding.addMethod("constructor", 11, &ProgramChangeCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<ProgramChange>); // clone
    binding.addMethod("_get", 4, &ProgramChangeGetters);
    binding.addMethod("_set", 4, &ProgramChangeSetters);
    binding.addMethod("_tostring", 9, &ProgramChangeToString);
    binding.endClass();

    // create class Midi.Sequence
    binding.startClass("Sequence", 8, SequenceObject);
    binding.addMethod("constructor", 11, &SequenceCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Sequence>); // clone
    binding.addMethod("_get", 4, &SequenceGetters);
    binding.addMethod("_nexti", 6, &SequenceNexti);
    binding.addMethod("add", 3, &Sequenceadd);
    binding.addMethod("clear", 5, &Sequenceclear);
    binding.addMethod("filter", 6, &Sequencefilter);
    binding.addMethod("len", 3, &Sequencelen);
    binding.addMethod("pattern", 7, &Sequencepattern);
    binding.addMethod("_tostring", 9, &SequenceToString);
    binding.endClass();

    // create class Midi.Sysex
    binding.startClass("Sysex", 5, SysexObject);
    binding.addMethod("constructor", 11, &SysexCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Sysex>); // clone
    binding.addMethod("_get", 4, &SysexGetters);
    binding.addMethod("_set", 4, &SysexSetters);
    binding.addMethod("_tostring", 9, &SysexToString);
    binding.endClass();

    // create class Midi.Tab
    binding.startClass("Tab", 3, TabObject);
    binding.addMethod("constructor", 11, &TabCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Tab>); // clone
    binding.addMethod("_get", 4, &TabGetters);
    binding.addMethod("_set", 4, &TabSetters);
    binding.addMethod("define", 6, &Tabdefine);
    binding.addMethod("sequence", 8, &Tabsequence);
    binding.addMethod("string", 6, &Tabstring);
    binding.addMethod("velocity", 8, &Tabvelocity);
    binding.addMethod("_tostring", 9, &TabToString);
    binding.endClass();

    // create class Midi.TempoChange
    binding.startClass("TempoChange", 11, TempoChangeObject);
    binding.addMethod("constructor", 11, &TempoChangeCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<TempoChange>); // clone
    binding.addMethod("_get", 4, &TempoChangeGetters);
    binding.addMethod("_set", 4, &TempoChangeSetters);
    binding.addMethod("_tostring", 9, &TempoChangeToString);
    binding.endClass();

    // create class Midi.Text
    binding.startClass("Text", 4, TextObject);
    binding.addMethod("constructor", 11, &TextCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Text>); // clone
    binding.addMethod("_get", 4, &TextGetters);
    binding.addMethod("_set", 4, &TextSetters);
    binding.addMethod("_tostring", 9, &TextToString);
    binding.endClass();

    // create class Midi.TimeSignature
    binding.startClass("TimeSignature", 13, TimeSignatureObject);
    binding.addMethod("constructor", 11, &TimeSignatureCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<TimeSignature>); // clone
    binding.addMethod("_get", 4, &TimeSignatureGetters);
    binding.addMethod("_set", 4, &TimeSignatureSetters);
    binding.addMethod("_tostring", 9, &TimeSignatureToString);
    binding.endClass();

    // create class Midi.TrackName
    binding.startClass("TrackName", 9, TrackNameObject);
    binding.addMethod("constructor", 11, &TrackNameCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<TrackName>); // clone
    binding.addMethod("_get", 4, &TrackNameGetters);
    binding.addMethod("_set", 4, &TrackNameSetters);
    binding.addMethod("_tostring", 9, &TrackNameToString);
    binding.endClass();

    sq_newtable(vm); // delegate table for package properties
    binding.addMethod("_newslot", 8, &MidiNewSlot);
    sq_setdelegate(vm, -2);

    binding.endTable(); // end package table
}
}}}
