#ifndef _ABCP__H_
#define _ABCP__H_

#include "dbg.h"
#include "yrx.h"
#include <stdlib.h>

/* .% Windows DLL definitions
  Only function declared as '|ABCAPI| will be exported
*/

#ifdef ABC_DLL
#include <windows.h>
#define ABCAPI __declspec(dllexport)
#else
#define ABCAPI
#endif


#define abcEmptyString ((char *)abcEMPTYSTR)
/* -- FUNCTIONS */

#define abcNote(_s,_f)         /* abcTNote        */ (_s)->val.note._f               
#define abcRest(_s,_f)         /* abcTRest        */ (_s)->val.rest._f               
#define abcSpacer(_s,_f)       /* abcTSpacer      */ (_s)->val.spacer._f             
#define abcTuplet(_s,_f)       /* abcTTuplet      */ (_s)->val.tuplet._f             
#define abcGChord(_s,_f)       /* abcTGChord      */ (_s)->val.gchord._f             
#define abcBroken(_s,_f)       /* abcTBroken      */ (_s)->val.broken._f             
#define abcAnnotation(_s,_f)   /* abcTAnnotation  */ (_s)->val.annotation._f         
#define abcDecoration(_s,_f)   /* abcTDecoration  */ (_s)->val.decoration._f         
#define abcLyrics(_s,_f)       /* abcTLyrics      */ (_s)->val.lyrics._f             
#define abcComment(_s,_f)      /* abcTComment     */ (_s)->val.comment._f            
#define abcSlur(_s,_f)         /* abcTSlur        */ (_s)->val.slur._f               
#define abcTie(_s,_f)          /* abcTSlur        */ (_s)->val.tie._f                
#define abcBar(_s,_f)          /* abcTBar         */ (_s)->val.bar._f                
#define abcEnding(_s,_f)       /* abcTEnding      */ (_s)->val.ending._f             
#define abcLength(_s,_f)       /* abcTFieldLength */ (_s)->val.field_L._f            
#define abcMeter(_s,_f)        /* abcTFieldMeter  */ (_s)->val.field_M._f            
#define abcTempo(_s,_f)        /* abcTFieldTempo  */ (_s)->val.field_Q._f            
#define abcKey(_s,_f)          /* abcTVK          */ (_s)->val.field_K._f            
#define abcVoice(_s,_f)        /* abcTVK          */ (_s)->val.field_V._f            
#define abcXField(_s,_f)       /* abcTXField      */ (_s)->val.Xfield._f             
#define abcField(_s,_f)        /* abcTFieldVoid   */ (_s)->val.field._f              
#define abcSpace(_s,_f)        /* abcTSpace       */ (_s)->val.space._f              
#define abcInclude(_s,_f)      /* abcTXInclude    */ (_s)->val.Xinclude._f           
#define abcText(_s,_f)         /* abcTText        */ (_s)->val.text._f               
#define abcMidi(_s,_f)         /* abcTXMidi       */ (_s)->val.midi._f               
#define abcDefine(_s,_f)       /* abcTDefine      */ (_s)->val.def._f 
#define abcPercussion(_s,_f)   /* abcTXDrum       */ (_s)->val.drum._f
#define abcRhythm(_s,_f)       /* abcTFieldRhythm */ (_s)->val.field_R._f            
#define abcRhythmTempo(_s,_f)  /* abcTFieldTempo  */ (_s)->val.field_R.tempo._f            
#define abcRhythmMeter(_s,_f)  /* abcTFieldMeter  */ (_s)->val.field_R.meter._f     
       
/* -- END FUNCTIONS */

#define abcStream(_s)        ((_s)->stream)
#define abcCursor(_s)        (abcStream(_s)->cursor)
#define abcState(_s)         ((_s)->state)
#define abcToken(_s)         ((_s)->token)
#define abcTokenLen(_s)      ((_s)->toklen)
#define abcTokenStart(_s)    ((_s)->tokstart)


typedef struct {
 /* I */   char            error   ;
 /* A */   unsigned char   pitch   ; /* A-G */
 /* I */   short           bend    ; /* */  
 /* A */   unsigned char   bending ; /* '\0' = no 'p' = cautionary 'b' = bend */
 /* I */   char            octave  ; /* -10 .. +10 */
 /* I */   unsigned short  duration;
} abcTNote;

typedef struct {
 /* I */ char error;
 /* A */ char type;
 /* I */ unsigned short  duration;
} abcTRest;

typedef struct {
 /* I */ char error;
 /* A */ char unit;
 /* I */ unsigned short space;
} abcTSpacer;

typedef struct {
 /* I */ char error;
 /* I */ unsigned char notes;
 /* I */ unsigned char time;
 /* I */ unsigned char span;
} abcTTuplet;

typedef struct {
 /* I */ char error;
 /* I */ unsigned char type;
 /* I */ unsigned char dots;
} abcTBroken;

typedef struct {
 /* I */ char error;
 /* I */ char pad;
 /* I */ unsigned short spaces;
 /* I */ unsigned short verse;
 /* $ */ unsigned char *syllable;
 /* - */ unsigned short syllable_len; 
} abcTLyrics;

typedef struct {
 /* I */ char error;
 /* A */ unsigned char  rootnote;
 /* A */ unsigned char  rootaccidental;
 /* A */ unsigned char  bassnote;
 /* A */ unsigned char  bassaccidental;
 /* $ */ unsigned char *string;
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *name;
 /* - */ unsigned short name_len;
 /* - */ unsigned char  name_buf[20];
} abcTGChord;

typedef struct {
 /* I */ char error;
 /* I */ unsigned char  position;
 /* I */ short          xoffset;
 /* I */ short          yoffset;
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *string;
} abcTAnnotation;

typedef struct {
 /* I */ char error;
 /* I */ unsigned char  type;
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *string;
} abcTComment;

typedef struct {
 /* I */ char error;
 /* A */ unsigned char type;
 /* A */ unsigned char direction;
 /* B */ unsigned char isDotted;
} abcTSlur;

typedef struct {
 /* I */ char error;
 /* I */ unsigned char  type;
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *string;
 /* I */ unsigned short ends;
 /* I */ unsigned short starts;
} abcTBar;

typedef struct {
 /* I */ char error;
 /* I */ unsigned char  num;
} abcTEnding;

typedef struct {
 /* I */ char error;
 /* I */ char           braces;
 /* & */ unsigned short code;
 /* I */ unsigned short spaces;
 /* I */ unsigned short verse;
 /* $ */ unsigned char *string;
 /* - */ unsigned short string_len;
 /* A */ unsigned char  class;
} abcTDecoration;

typedef struct {
 /* I */ char error;
 /* A */ unsigned char  type;
 /* I */ unsigned short spaces;
} abcTSpace;

typedef struct {
 /* I */ char error;
 /* A */ unsigned char  type;
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *string;
} abcTText;


typedef struct {
 /* I */ char error;
 /* A */ char field; 
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *string;
} abcTFieldVoid;

typedef struct {
 /* I */ char error;
 /* A */ char field;
 /* - */ unsigned short  string_len;
 /* $ */ unsigned char  *string;
 /* I */ unsigned short  duration;
} abcTFieldLength;

typedef struct {
 /* I */ char error;
 /* A */ char field;
 /* - */ unsigned short string_len;
 /* $ */ unsigned char *string;
 /* I */ unsigned char  num;
 /* I */ unsigned char  den;
 /* I */ unsigned char  playNum;
 /* I */ unsigned char  playDen;
 /* I */ unsigned char  brace;  
 /* I */ unsigned char  numAdds[11];
 /* B */ unsigned char  isCommon;
 /* B */ unsigned char  isCut;
 /* B */ unsigned char  isFree;
} abcTFieldMeter;

typedef struct {
 /* I */ char             error;
 /* A */ char             field;
 /* - */ unsigned short   string_len;
 /* $ */ unsigned char   *string;
 /* $ */ unsigned char   *mark;
 /* - */ unsigned short   mark_len;
 /* I */ unsigned short   code;
 /* I */ unsigned short   beats;  
 /* I */ unsigned short   note;
 /* I */ unsigned short   notes[5];
 /* B */ unsigned char    isRelative;
} abcTFieldTempo;

#define ABC_RHYTHM_MAXPAIRS 16

typedef struct {
 /* I */ char             error;
 /* A */ char             field;
 /* - */ unsigned short   string_len;
 /* $ */ unsigned char   *string;
 /* - */ unsigned short   name_len;
 /* $ */ unsigned char   *name;
 /* - */ abcTFieldMeter   meter;
 /* - */ abcTFieldTempo   tempo;
 /* B */ unsigned char    hasMeter;
 /* B */ unsigned char    hasTempo;
 /* I */ unsigned short   pairs;
 /* I */ unsigned char    velocity[16];
 /* I */ unsigned char    duration[16]; 
} abcTFieldRhythm;

typedef struct {
 /* I */ char            error;
 /* A */ char            field;
 /* - */ unsigned short  string_len;
 /* $ */ unsigned char  *string;
 /* A */ unsigned char   tonic;
 /* A */ unsigned char   accidentals;
 /* A */ unsigned char   mode;
 /* A */ unsigned char   signature[7];
 /* $ */ unsigned char  *id;
 /* - */ unsigned short  id_len;
 /* $ */ unsigned char  *name;
 /* - */ unsigned short  name_len;
 /* $ */ unsigned char  *shortName;
 /* - */ unsigned short  shortName_len;
 /* $ */ unsigned char  *instr;
 /* - */ unsigned short  instr_len;
 /* & */ unsigned short  instrCode;
 /* $ */ unsigned char  *head;
 /* - */ unsigned short  head_len;
 /* & */ unsigned short  headCode;
 /* & */ unsigned short  clef;
 /* I */ unsigned char   clefline;
 /* I */ char            stvGrouped;
 /* A */ unsigned char   stvGroupType;
 /* I */ char            stvSpace;
 /* I */ char            octaves; 
 /* I */ char            transpose;
 /* I */ unsigned char   staffLines;
 /* I */ unsigned char   middlePitch;
 /* I */ char            middleOctave;
 /* A */ char            stems;
 /* A */ char            gStems;
 /* A */ char            marks;
 /* A */ char            lyrics;
} abcTVK;

#define abcTFieldKey   abcTVK
#define abcTFieldVoice abcTVK
#define abcTClef abcTVK

typedef struct {
  char error;
} abcTVal;

typedef struct {
 /* I */ char error;
 /* - */ char pad;
 /* & */ unsigned short code;
 /* $ */ unsigned char *key;
 /* - */ unsigned short key_len;
 /* $ */ unsigned char *value;
 /* - */ unsigned short value_len;
} abcTXField;

typedef struct {
 /* I */ char error;
 /* - */ char pad;
 /* I */ unsigned short code;
 /* $ */ unsigned char *key;
 /* - */ unsigned short key_len;
 /* $ */ unsigned char *filename;
 /* - */ unsigned short filename_len;
} abcTXInclude;

typedef struct {
 /* I */ char             error;
 /* I */ unsigned char    midiPitch;
 /* & */ unsigned short   instrCode;
 /* $ */ unsigned char   *instr;
 /* - */ unsigned short   instr_len;
 /* & */ unsigned short   headCode;
 /* $ */ unsigned char   *head;
 /* - */ unsigned short   head_len;
 /* A */ unsigned char    notePitch;
 /* A */ unsigned char    accidentals; /* FfnNsS */
 /* I */ unsigned char    octave;
 /* A */ unsigned char    stem; /* UDN */
} abcTXDrum;

typedef struct {
 /* I */ char             error;
 /* I */ unsigned char    argc;
 /* & */ unsigned short   cmd;
 /* $ */ unsigned char   *cmdstr;
 /* - */ unsigned short   cmdstr_len;
 /* $ */ unsigned char   *argsA;
 /* - */ unsigned short   argsA_len;
 /* $ */ unsigned char   *argsB;
 /* - */ unsigned short   argsB_len;
 /* I */          char    argv[20];
} abcTXMidi;

typedef struct {
 /* I */ char error;
 /* A */ unsigned char   type;
 /* $ */ unsigned char  *term;
 /* - */ unsigned short  term_len;
 /* $ */ unsigned char  *definition;
 /* - */ unsigned short  definition_len;
} abcTDefine;

typedef  union {
  abcTVal          val;
  abcTNote         note;         
  abcTRest         rest;         
  abcTSpacer       spacer;       
  abcTSpace        space;        
  abcTTuplet       tuplet;       
  abcTGChord       gchord;       
  abcTBroken       broken;       
  abcTAnnotation   annotation;   
  abcTDecoration   decoration;   
  abcTLyrics       lyrics;       
  abcTComment      comment;      
  abcTSlur         slur;         
  abcTSlur         tie;          
  abcTBar          bar;          
  abcTEnding       ending;       
  abcTFieldVoid    field;        
  abcTFieldLength  field_L;      
  abcTFieldMeter   field_M;      
  abcTFieldTempo   field_Q;      
  abcTFieldKey     field_K;      
  abcTFieldVoice   field_V;
  abcTFieldRhythm  field_R;      
  abcTXField       Xfield;       
  abcTXInclude     Xinclude;
  abcTXMidi        midi;     
  abcTText         text; 
  abcTDefine       def;      
  abcTXDrum        drum;  
}  abcTValues;            

/* 960 = 2^6 * 3 * 5 */
#define abcTICKS 960

#define MAX_NESTED_INC 8

typedef struct {
  unsigned short   _guard;
  unsigned short   token;
  unsigned short   state;
  unsigned short   nextstate;
  unsigned short   oldstate;
  unsigned short   toklen;
  unsigned char   *tokstart;
  unsigned short   stream_ndx;
  unsigned short   states[MAX_NESTED_INC];
  YYSTREAM        *streams[MAX_NESTED_INC];
  YYSTREAM        *stream;
  abcTValues       val;
} abcScanner;


/* .% Scanner states
*/
#define S_EOF                 0x0000
#define S_ERROR               0x0001
#define S_LIMBO               0x0002
#define S_NONE                0x0003
#define S_EATSOL              0x0004
#define S_TUNE                0x8000
#define S_TUNE_G              0x8001  /* in grace */
#define S_TUNE_C              0x8002  /* in a chord */
#define S_FIELD               0x1000  
#define S_EXTFIELD            0x2000
#define S_INFIELD             0x4000
#define S_CLEARIN             0xBFFF  /* */
#define S_ANYFIELD            0x7000
#define S_CFIELD              0x5000  /* A "colon" field like A: or [A:]*/
#define S_INCLUDE             0x0010
#define S_TEXT                0x0011

/* -- BEGIN TOKENS  */

#define T_FIRST               0x0000 
#define T_UNKNOWN             0x0001 
#define T_NONE                0x0002 
#define T_EOF                 0x0003 
#define T_COMMENT             0x0005 
#define T_EMPTYLINE           0x0006 
#define T_EXTFIELD            0x0007 
#define T_FIELD               0x0008 
#define T_NOTE                0x0009 
#define T_SPACE               0x000A 
#define T_BAR                 0x000B 
#define T_INFIELD             0x000C 
#define T_TEXT                0x000D 
#define T_ENDLINE             0x000E 
#define T_CONTINUE            0x000F 
#define T_DECORATION          0x0010 
#define T_REST                0x0011 
#define T_INVREST             0x0012 
#define T_MULTIREST           0x0013 
#define T_SPACER              0x0014 
#define T_TUPLET              0x0015 
#define T_SLURSTART           0x0016 
#define T_SLUREND             0x0017 
#define T_BROKENRIGHT         0x0018 
#define T_BROKENLEFT          0x0019 
#define T_CHORDSTART          0x001A 
#define T_CHORDEND            0x001B 
#define T_GCHORD              0x001C 
#define T_REPEAT              0x001D 
#define T_ACCIACATURA         0x001E 
#define T_APPOGGIATURA        0x001F 
#define T_GRACEEND            0x0020 
#define T_TIE                 0x0021 
#define T_STRING              0x0022 
#define T_STRINGUNF           0x0023 
#define T_ANNOTATION          0x0024 
#define T_BREAK               0x0025 
#define T_STARTLINE           0x0026 
#define T_FIELDB              0x0027 
#define T_PRAGMA              0x0028 
#define T_INCLUDE             0x0029 
#define T_INCLUDE_EOF         0x002A 
#define T_ENDING              0x002B 
                                     
#define T_XFIELD              0x002D 
                                     
#define T_LYR_SYLLABLE        0x002F 
#define T_LYR_BLANK           0x0030 
#define T_LYR_BAR             0x0031 
#define T_LYR_SPACE           0x0032 
#define T_LYR_HOLD            0x0033 
#define T_LYR_VERSE           0x0034 
#define T_LYR_CONTINUE        0x0035 
                                     
#define T_TIE_DOT             0x0037 
#define T_DUPMESURE           0x0038 
                                     
#define T_SYM_BLANK           0x003A 
#define T_SYM_BAR             0x003B 
#define T_SYM_VERSE           0x003C 
#define T_SYM_CONTINUE        0x003D 
#define T_SYM_GCHORD          0x003E 
#define T_SYM_DECORATION      0x003F 
#define T_SYM_ANNOTATION      0x0040 
                                     
#define T_BSPACE              0x0041 
#define T_DOTLEFT2            0x0042 
#define T_DOTRIGHT2           0x0043 
#define T_LAST                0x0044 
                                     
#define T_OVERLAY             0x0046 
#define T_OVLEND              0x0047 
#define T_OVLSTART            0x0048 
#define T_DOVERLAY            0x0049 
#define T_DOVLEND             0x004A 
#define T_DOVLSTART           0x004B 

#define T_BEGINTEXT           0x0050
#define T_ENDTEXT             0x0051   

#define T_MIDI                0x0052
#define T_DEFINE              0x0053
#define T_SETDRUM             0x0054
                          
/* -- END TOKENS  */

#define abc_isInField(_s)   ((_s->state) & S_INFIELD)
#define abc_setInField(_s)  ((_s->state) |= S_INFIELD)
#define abc_clrInField(_s)  ((_s->state) &= ~S_INFIELD)

int abc_isFieldEnd(abcScanner *s) ;
void abc_gotoFieldEnd(abcScanner *s);

#define abc_FieldEnd(_s)    (abc_isInField(_s)?']':YYEOL)

#define abc_SetToken(s,t)        ((s)->toklen=YYLEN(0),(s)->tokstart=YYSTART(0),(s)->token=t)
#define abc_SetEmptyToken(s,t)   ((s)->toklen=0,(s)->tokstart=abcEmptyString,(s)->token=t)
                          
#define abc_eatSOL(s) do {if (*((s)->cursor) == '\0')  yygetline(s);\
                          if (*((s)->cursor) == YYSOL) (s)->cursor +=1;} while (0)

#define abc_ZEROVAL(a)  memset(&((a)->val),0,sizeof(abcTValues))


int abcSField      (abcScanner *s);
int abcSLimbo      (abcScanner *s);
int abcSTune       (abcScanner *s);
int abcSExtField   (abcScanner *s);
int abcSWords      (abcScanner *s);
int abcSSymbols    (abcScanner *s);
int abcSText       (abcScanner *s);
                   
int abc_MIDI       (abcScanner *s);
int abc_Note       (abcScanner *s);
int abc_Chord      (abcScanner *s,unsigned char *start, unsigned short length);
int abc_Rest       (abcScanner *s,char *start);
int abc_Field      (abcScanner *s,unsigned char *start, char field,unsigned short tok);
int abc_Annotation (abcScanner *s);
int abc_Bar        (abcScanner *s, unsigned char *st,unsigned short ln);
int abc_setdrum    (abcScanner *s);

YYSTREAM *abc_addstream (abcScanner *s, unsigned char *filename, 
                                        unsigned char *string,
                                        unsigned long bufsize);
YYSTREAM *abc_delstream (abcScanner *s);

#define abc_addfilestream(_s,_f,_n)  abc_addstream(_s,_f,NULL,_n)
#define abc_addstringstream(_s,_t)   abc_addstream(_s,NULL,_t,0)


#ifdef ABC_USEDLL
#define _L (*
#define _R )
#else 
#define _L ABCAPI
#define _R 
#endif


unsigned short  _L abc_ChkString    _R  (char *str);
unsigned short  _L abc_ChkNString   _R  (char *str,int len);
char          * _L abc_RevString    _R  (unsigned short code);

const char    * _L abcTokString     _R  (abcScanner *s);

unsigned short  _L abcNextToken     _R  (abcScanner *s);
abcScanner    * _L abcFileScanner   _R  (char *filename,unsigned long bufsize);
abcScanner    * _L abcStringScanner _R  (char *string);
void            _L abcClose         _R  (abcScanner *s);
unsigned int    _L abcLineNum       _R  (abcScanner *s);
const char    * _L abcFileName      _R  (abcScanner *s);
void          * _L abcVal           _R  (abcScanner *s);
int             _L abcStopInclude   _R  (abcScanner *s);


#define abcIsScanner(_s) ((_s)->_guard == 0xF00D)

#ifndef _ABC_STRINGS_H
#include "abcp_strings.h"
#endif

#ifdef NOEXTERN
  const char *abcEMPTYSTR="";
  unsigned char cl[ABC_MAXLEN_STR+10];
#else
  extern const char *abcEMPTYSTR;
  extern unsigned char cl[];
#endif



#if defined(ABC_STR_INSTRUMENT)
unsigned short abc_getinstrument(unsigned char *i, unsigned short l);
#else
  #define abc_getinstrument(i,l) 0
#endif


#endif /*_ABCP__H_ */
