#line 1 "key.yc"

#include "abcp_.h"


/*

  M Major
  m Minor
  X Mix
  D Dor
  P Phr
  L Lyd
  l Loc
  I Ion
  A Aeo
  E Explicit


1= Missing Tonic
2= Unknown mode
3=
4=
5=
6= Syntax error



 first string length
 n=first is name, s=first is subname
 stem direction  0100 ggss  00=auto; 11=none; 10=up; 01=down;
 Middle (pitch and octave on the middle line [A-G] default B)
 Staff lines (0 .. 9)
 Semitones transpose (-128 .. +127)
 Octave transpose 'O'=0
 clef name + center line
   40 none         (none)
   4F percussion   (perc)
   4F perc         (perc)
   41 frenchviolin (treble1)
   41 french       (treble1)
   4A violin       (treble)
   4A treble       (treble)
   51 soprano      (alto1)
   52 mezzosoprano (alto2)
   5A alto         (alto)
   54 tenor        (alto4)
   55 baritone     (alto5)
   6A bass         (bass)
   65 subbass      (bass5)

   71 doh1 (Gregorian)
   72 doh2
   73 doh3
   74 doh4
   75 Fa1  (Gregorian)
   76 Fa2
   77 Fa3
   78 Fa4

                line
   treble  0100 xxxx
   alto    0101 xxxx
   bass    0110 xxxx
   doh     0111 0xxx
   fa      0111 1xxx

   30

If line is 0 the clef is invisible (i.e. "none" is "treble0").
if line is A (10) no line value has been specified

Staff lines
  The parser handles up to 9 staff lines numbered from 1
to 9, the position of the clef and/or the the "middle="
parameters determine the note on each line.

Stems

 0100 ggss  00=auto; 11=none; 10=up; 01=down;

Errors:
1= invalid note
2=
3=
4= Unrecognized token
5=
6=
*/


static char getdirection(abcScanner *s)
{
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Cu\a+" )
        ":RuG/G\0\0",
_Y_(  2,"\Cd\a+" )
        ":RdG/G\0\0",
_Y_(  3,"\Ca\a+" )
        ":RaG/G\0\0",
_Y_(  4,"\Cn\a+" )
        ":RnG/G\0\0",
    NULL};
#line 93
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 : /* up    */
    case 2 : /* down  */
    case 3 : /* auto  */
    case 4 : /* none  */
               return toupper(*YYSTART(0));
  }} /* abcStream(s */

  return 0;
}

static unsigned short getclef(abcScanner *s, unsigned short d)
{

{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"[CFGP]\>" )
        ":\210\12\310\0\1?\0",
    NULL};
#line 106
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 :
        switch (*YYSTART(0)) {
          case 'G': d = 0x504A; break;
          case 'F': d = 0x506A; break;
          case 'C': d = 0x505A; break;
          case 'P': d = 0x5250; break;
        }
  }} /* abcStream(s */

  return d ;
}

static int getnum(abcScanner *s)
{
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\N" )
        ":#\0",
    NULL};
#line 120
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 : return (atoi(YYSTART(0)));
  }} /* abcStream(s */

  return 0;
}

static int getunum(abcScanner *s)
{
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\d+" )
        ":D/D\0\0",
    NULL};
#line 128
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 : return (atoi(YYSTART(0)));
  }} /* abcStream(s */

  return 0;
}


#ifdef ABC_STR_PRINT
static void getnotehead(abcScanner *s, abcTClef *clef)
{
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Y(\W)" )
        ":(<\1)>\1\0",
    NULL};
#line 138
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 : *cl = '%';
               YYSTRNCPY(cl+1,1,ABC_MAXLEN_STR);
               clef->headCode = abc_ChkString(cl);
               clef->head     = YYSTART(1);
               clef->head_len = YYLEN(1);
               if ((clef->headCode & 0xFFF0) == 0x5EF0)
                 clef->headCode &= 0x000F;
               else
                 clef->headCode = 0x0000;
               break;
    default:
               clef->headCode = 0;
               clef->head_len = 0;
               clef->head     = NULL;
               break;
  }} /* abcStream(s */

}

#else
#define getnotehead(s, c)
#endif

static void getmiddlenote(abcScanner *s, abcTClef *clef)
{
  char *t;
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"[A-Ga-g]([',]*)\s*" )
        ":\210\14\376\0\0\0\376<\1/\204\5\200\20\0>\1/F\0\0",
    NULL};
#line 164
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 :
        clef->middlePitch=*YYSTART(0);
        clef->middleOctave=4;
        if (islower(clef->middlePitch)) {
          clef->middleOctave++;
          clef->middlePitch=toupper(clef->middlePitch);
        }

        if (YYLEN(1) > 0) {
          t=YYSTART(1);
          while (t<YYEND(0)) {
            if (*t == ',') clef->middleOctave--;
            else clef->middleOctave++;
            t++;
          }
        }
  }} /* abcStream(s */


}

static void getstring(abcScanner *s,unsigned char **pstr, unsigned short *plen)
{
  unsigned short n=0;
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Q" )
        ":!\0",
_Y_(  2,"[^\0\c\s]+" )
        ":\201\341 \377/\201\341 \377\0\0",
    NULL};
#line 188
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case 1 : n=1;
    case 2 :
           *pstr=YYSTART(0)+n;
           *plen=YYLEN(0)-n*2;
           /*fprintf(stderr,"++%c++",**pstr);*/
           break;
  }} /* abcStream(s */

}

static int clef(abcScanner *s, abcTClef *clef)
{
  int d;

  char *t,*q;   int i;
  unsigned char u=0;

  YYSTATE(get_clef) {
    if (*(abcCursor(s)) == abc_FieldEnd(s)) return 0;

{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Y[+-]\d+" )
        ":(\205\5(D/D\0\0",
_Y_(  2,"\Y(\a+)\Y=?\Y" )
        ":(<\1G/G\0>\1(-Q=\0(\0",
_Y_(  3,".\Y" )
        ":J(\0",
    NULL};
#line 208
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
      case -1 : return -1;

      case 1 : i=atoi(YYSTART(0));
                    switch(i) {
                      case   8: clef->octaves =  1; break;
                      case  -8: clef->octaves = -1; break;
                      case  15: clef->octaves =  2; break;
                      case -15: clef->octaves = -2; break;
                      case  22: clef->octaves =  3; break;
                      case -22: clef->octaves = -3; break;
                    }
                    /*fprintf(stderr,"octaves=%d\n",i);fflush(stderr);*/
                    break;

      case 2 :
         cl[0]='%'; i=1;
         t=YYSTART(1);q=YYEND(1);
         while (t<q && i<31) cl[i++]=*t++;
         cl[i]='\0';
         d=abc_ChkString(cl);

         /*fprintf(stderr,"--%s--[%04x]\n",cl,d);*/
         if (d == 0x5EF0) {  /* note head */
           getnotehead(s,clef);
         }
         else {
           if (d == 0x5E33) d = getclef(s,d);
           switch (d & 0xFF00 ) {
               case 0x5100 :
               case 0x5200 : clef->clef = d;
                             break;

               case 0x5000 : clef->clef = d;
                             d = clef->clef & 0x000F;
                             if (d == 0x0A) {
                               u=getunum(s);
                               if (u != 0) d=u;
                             }
                             if (d > 9) {
                               u=clef->clef & 0x00F0;
                               switch (u) {
                                 case 0x40: clef->clefline = 2; break; /* G */
                                 case 0x50: clef->clefline = 3; break; /* C */
                                 case 0x60: clef->clefline = 4; break; /* F */
                                 case 0x70: clef->clefline = 2; break; /* Doh */
                                 case 0x80: clef->clefline = 3; break; /* Fa */
                               }
                             }
                             /*fprintf(stderr, "-- clef= %04x\n",clef->clef);*/
                             break;


               case 0x5E00 : switch (d & 0x00FF) {
              /* staves    */  case 0x21 :
              /* bracket   */  case 0x22 :
              /* brace     */  case 0x23 : clef->stvGrouped   = (unsigned char)getnum(s);
              /* merge     */  case 0x24 : clef->stvGroupType = " |[{="[d & 0x0F];
                                           break;
              /* space     */  case 0x2A : clef->stvSpace   = (char)getnum(s); break;
              /* octave    */  case 0x2B : clef->transpose += ((char)getnum(s)*12); break;
              /* transpose */  case 0x2C : clef->transpose += (char)getnum(s); break;
              /* stafflines*/  case 0x2D : clef->staffLines = (unsigned char)getnum(s); break;
              /* middle    */  case 0x2E : getmiddlenote(s,clef); break;
              /* gstem     */  case 0x2F : clef->gStems = getdirection(s); break;
              /* stem      */  case 0x30 : clef->stems  = getdirection(s); break;
              /* dyn marks */  case 0x35 : clef->marks  = getdirection(s); break;
              /* lyr pos   */  case 0x36 : clef->lyrics = getdirection(s); break;
              /* name      */  case 0x31 : getstring(s,&clef->name, &clef->name_len); break;
              /* subname   */  case 0x32 : getstring(s,&clef->shortName, &clef->shortName_len); break;
              /* instrument*/  case 0x34 : getstring(s,&clef->instr, &clef->instr_len);
                                           clef->instrCode=abc_getinstrument(clef->instr, clef->instr_len);
                                           break;
                             }
                             break;
           }
       }

      case 3 : break;

    }} /* abcStream(s */

    YYGOTO(get_clef);
  }
  return 0;
}

int abc_FieldKey(abcScanner *s)
{
  char t;
  int m=0;

  /* .{ Get the Tonic */
  if (*abcCursor(s) == abc_FieldEnd(s)) return 0;

{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Y([A-Ga-g])([#b]?)\>" )
        ":(<\1\210\14\376\0\0\0\376>\1<\2-\204\14\10\0\0\0\0\0\0\0\4\0>\2?\0",
_Y_(  2,"\Y\C(hp)" )
        ":(<\1Thp\0>\1\0",
_Y_(  3,"\Y\Cnone" )
        ":(Tnone\0\0",
    NULL};
#line 302
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case -1 : s->nextstate = S_EOF;
                        YYGOTO(done);


    case 1 : t='n';
                        if (YYLEN(2)> 0) {
                          switch (*YYSTART(2)) {
                            case '#' : t='s'; break;
                            case 'b' : t='f'; break;
                          }
                          if ((YYLEN(2)> 1) && (YYSTART(2)[1] == t))
                            t=toupper(t);
                        }
                        s->val.field_K.accidentals=t;
                        s->val.field_K.tonic = toupper(*YYSTART(1));
                        break;

    case 2 : s->val.field_K.tonic = YYSTART(1)[1]; break;
    case 3 : s->val.field_K.tonic = 'Z'; break;
  }} /* abcStream(s */

  /* .} */

  /* .{ Get the mode */
  YYSTATE(mode) {
 if (*abcCursor(s) == abc_FieldEnd(s)) return 0;
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\s\Y" )
        ":F(\0",
_Y_(  2,"\Cmaj[\a]*" )
        ":Tmaj\0/\237\237@\0\0\0",
_Y_(  3,"\Cmin[\a]*" )
        ":Tmin\0/\237\237@\0\0\0",
_Y_(  4,"\Cmix[\a]*" )
        ":Tmix\0/\237\237@\0\0\0",
_Y_(  5,"\Cdor[\a]*" )
        ":Tdor\0/\237\237@\0\0\0",
_Y_(  6,"\Cphr[\a]*" )
        ":Tphr\0/\237\237@\0\0\0",
_Y_(  7,"\Clyd[\a]*" )
        ":Tlyd\0/\237\237@\0\0\0",
_Y_(  8,"\Cloc[\a]*" )
        ":Tloc\0/\237\237@\0\0\0",
_Y_(  9,"\Cion[\a]*" )
        ":Tion\0/\237\237@\0\0\0",
_Y_( 10,"\Caeo[\a]*" )
        ":Taeo\0/\237\237@\0\0\0",
_Y_( 11,"\Cexp[\a]*" )
        ":Texp\0/\237\237@\0\0\0",
_Y_( 12,"m" )
        ":Qm\0",
_Y_( 13,"M" )
        ":QM\0",
    NULL};
#line 328
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
      case -1 : s->nextstate = S_EOF;
                   YYGOTO(done);

      case 1 : YYGOTO(mode);

      case 2 :  s->val.field_K.mode ='M' ; break;
      case 3 :  s->val.field_K.mode ='N' ; break;
      case 4 :  s->val.field_K.mode ='X' ; break;
      case 5 :  s->val.field_K.mode ='D' ; break;
      case 6 :  s->val.field_K.mode ='P' ; break;
      case 7 :  s->val.field_K.mode ='Y' ; break;
      case 8 :  s->val.field_K.mode ='L' ; break;
      case 9 :  s->val.field_K.mode ='I' ; break;
      case 10 :  s->val.field_K.mode ='A' ; break;
      case 11 :  s->val.field_K.mode ='E' ; break;
      case 12 :  s->val.field_K.mode ='N' ; break;
      case 13 :  s->val.field_K.mode ='M' ; break;
    }} /* abcStream(s */

  }
  /* .} */

  /* .{ Get explicit accidentals in the key */
  YYSTATE(exp) {
 if (*abcCursor(s) == abc_FieldEnd(s)) return 0;
    m=0;
{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Y([A-Ga-g])##" )
        ":(<\1\210\14\376\0\0\0\376>\1S##\0\0",
_Y_(  2,"\Y^^([A-Ga-g])" )
        ":(S^^\0<\1\210\14\376\0\0\0\376>\1\0",
_Y_(  3,"\Y([A-Ga-g])#" )
        ":(<\1\210\14\376\0\0\0\376>\1Q#\0",
_Y_(  4,"\Y^([A-Ga-g])" )
        ":(Q^<\1\210\14\376\0\0\0\376>\1\0",
_Y_(  5,"\Y([A-Ga-g])bb" )
        ":(<\1\210\14\376\0\0\0\376>\1Sbb\0\0",
_Y_(  6,"\Y__([A-Ga-g])" )
        ":(S__\0<\1\210\14\376\0\0\0\376>\1\0",
_Y_(  7,"\Y([A-Ga-g])b" )
        ":(<\1\210\14\376\0\0\0\376>\1Qb\0",
_Y_(  8,"\Y_([A-Ga-g])" )
        ":(Q_<\1\210\14\376\0\0\0\376>\1\0",
_Y_(  9,"\Y([A-Ga-g])=" )
        ":(<\1\210\14\376\0\0\0\376>\1Q=\0",
_Y_( 10,"\Y=([A-Ga-g])" )
        ":(Q=<\1\210\14\376\0\0\0\376>\1\0",
    NULL};
#line 354
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
      case -1 : s->nextstate = S_EOF;
                          break;

      case 1 :
      case 2 : m='S'; break;

      case 3 :
      case 4 : m='s'; break;

      case 5 :
      case 6 : m='F'; break;

      case 7 :
      case 8 : m='f'; break;

      case 9 :
      case 10 : m='N'; break;

    }} /* abcStream(s */

    if (m != 0) {
      s->val.field_K.signature[((*YYSTART(1)) & 0x07)-1] = m;
      YYGOTO(exp);
    }
  }
  /* .} */
  YYSTATE(done) {
 s->nextstate = S_TUNE;   /* K: marks the end of header */
    return clef(s,&s->val.field_K);
  }
}

int abc_FieldVoice(abcScanner *s)
{
  unsigned char n=0;

  if (*abcCursor(s) == abc_FieldEnd(s)) return 0;

{ /* abcStream(s */
    static const unsigned char *_Y[] = {
_Y_(  1,"\Y(\Q)\Y" )
        ":(<\1!>\1(\0",
_Y_(  2,"\Y([-.+\a\d]+)\Y" )
        ":(<\1\205\205Hh/\205\205Hh\0>\1(\0",
    NULL};
#line 392
    switch (yyswitch((unsigned char **)_Y,abcStream(s)) ) {
    case -1 : s->nextstate = S_EOF;
                        return 0;

    case 1 : n=1   ;
    case 2 : abcVoice(s,id) = YYSTART(1) +n;
                           abcVoice(s,id_len) = YYLEN(1) - (2*n);
                          break;
  }} /* abcStream(s */


  YYSTATE(done) {
    return clef(s,&s->val.field_V);
  }
}

