#define DBG_FILENAME "00_abcp.log"
#define DBG_MAIN
#include "abcp_.h"

void ABCAPI abcClose(abcScanner *s)
{
  YYSTREAM *ys;
  if (s) {
    while (s->stream_ndx > 0) {
      ys=s->streams[--s->stream_ndx];
      if (ys) YYCLOSE(ys);
    }
    s->stream=NULL;
    s->_guard=0xBAAD;
    free(s);
  }
}

YYSTREAM *abc_addstream(abcScanner *s, unsigned char *filename,unsigned char *string,unsigned long bufsize)
{
  YYSTREAM *ys=NULL;
  
  if (s->stream_ndx < 8) {
    if (string != NULL) 
      ys=YYSTRING(string);
    else
      ys=YYFILE(filename,bufsize);
      
    if (ys) {
      s->states[s->stream_ndx]=s->state;
      /*printf("++%x++\n",s->states[s->stream_ndx]);*/
      s->streams[s->stream_ndx++]=ys;
      s->stream = ys;
    }
  }
  return ys;
}

YYSTREAM *abc_delstream(abcScanner *s)
{
  YYSTREAM *ys=NULL;
  if (s) {
    if (s->stream_ndx > 1) {
      s->stream_ndx--;
      s->state=s->states[s->stream_ndx];
      ys=s->streams[s->stream_ndx];
      if (ys) YYCLOSE(ys);
      s->streams[s->stream_ndx] = NULL;
      s->stream =s->streams[s->stream_ndx-1];
      s->nextstate=S_NONE;
      s->token=T_INCLUDE_EOF;
      ys=s->stream;
    }
  }
  
  return ys;
}

abcScanner * ABCAPI abcFileScanner(char *filename,unsigned long bufsize)
{
  abcScanner *s=NULL;
  
  s=malloc(sizeof(abcScanner));
  if (s) {  
    s->_guard = 0xF00D;
    s->stream_ndx = 0;
    s->state  = S_LIMBO;
    s->nextstate=S_NONE;
    s->oldstate=S_NONE;
    s->token=T_UNKNOWN;
    s->tokstart=NULL;
    s->toklen=0;
    if (abc_addfilestream(s,filename,bufsize) == NULL) {
      free(s);
      s=NULL;
    }
  }                  
  return s;
}


abcScanner * ABCAPI abcStringScanner(char *string)
{
  abcScanner *s=NULL;
  
  if (string == NULL || *string == '\0') return NULL;
  
  s=malloc(sizeof(abcScanner));
  if (s) {  
    s->_guard = 0xF00D;
    s->stream_ndx = 0;
    s->state  = S_LIMBO;
    s->nextstate=S_NONE;
    s->oldstate=S_NONE;
    s->token=T_UNKNOWN;
    s->tokstart=NULL;
    s->toklen=0;
    if (abc_addstringstream(s,string) == NULL) {
      free(s);
      s=NULL;
    }
  }                  
  return s;
}

const char * ABCAPI abcFileName(abcScanner *s)
{
  return YYFILENAME(abcStream(s));
}

unsigned int ABCAPI abcLineNum(abcScanner *s)
{
  return YYLINENUMBER(abcStream(s));
}

