/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BEATTRACKER_H
#define BEATTRACKER_H

#include "BTrack.h"

#include "audioengine.h"
#include "audioport.h"
#include "basicclock.h"
#include "midiport.h"

namespace bipscript {

const unsigned int BT_HOP_SIZE = 512;

class BaseBeatTracker : public time::BasicClock {
  protected:
    BTrack btrack;

  public:
    BaseBeatTracker(float bpm) : time::BasicClock(bpm) {}
    void tick(samples_t time, bool beat, samples_t period);

  private:
    double closestBeat(double location);
};

namespace audio {

class BeatTracker : public BaseBeatTracker {
    double *btbuffer;
    samples_t index;
    AudioInputPort &minput;

  public:
    BeatTracker(AudioInputPort &input, float bpm) : BaseBeatTracker(bpm), index(0), minput(input) {
        btbuffer = new double[BT_HOP_SIZE];
    }
    bool hasHistory() override { return false; }
    void update(samples_t time) override;
    void recycle() override {
        // TODO: implement
    }
};

class BeatTrackerFactory {
  public:
    static BeatTrackerFactory &instance() {
        static BeatTrackerFactory instance;
        return instance;
    }
    BeatTracker *newBeatTracker(AudioInputPort &input, float bpm) {
        BeatTracker *bt = new BeatTracker(input, bpm);
        AudioEngine::instance().addClock(bt);
        return bt;
    }
};

} // end namespace audio

namespace midi {

class BeatTracker : public BaseBeatTracker {
    BTrack btrack;
    samples_t index;
    double onsetSample;
    float noteWeight[128];
    MidiInputPort &minput;

  public:
    BeatTracker(MidiInputPort &input, float bpm)
        : BaseBeatTracker(bpm), index(0), onsetSample(0.0), minput(input) {
        // reset note weights
        for (int i = 0; i < 128; i++) {
            noteWeight[i] = 1.0;
        }
    }
    void update(samples_t) override;
};

class BeatTrackerFactory {
  public:
    static BeatTrackerFactory &instance() {
        static BeatTrackerFactory instance;
        return instance;
    }
    BeatTracker *newBeatTracker(MidiInputPort &input, float bpm) {
        BeatTracker *bt = new BeatTracker(input, bpm);
        AudioEngine::instance().addClock(bt);
        return bt;
    }
};

} // namespace midi
} // namespace bipscript
#endif // BEATTRACKER_H
