/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ONSETDETECTOR_H
#define ONSETDETECTOR_H

#include "OnsetDetectionFunction.h"
#include "audiosink.h"
#include "processorcache.h"

namespace bipscript {
namespace audio {

const unsigned int ONSET_HOP_SIZE = 512;
const unsigned int HISTORY_SIZE = 8;

class OnsetDetector : public Sink, public AudioProcessor {
    OnsetDetectionFunction odf;
    unsigned int index;
    double *buffer;
    double history[HISTORY_SIZE];
    float thold;
    samples_t lastOnsetFrame;
    std::atomic<ScriptFunction *> onOnsetHandler;

  public:
    OnsetDetector();
    ~OnsetDetector();
    void onOnset(ScriptFunction &handler);
    float threshold() { return thold; }
    float threshold(float threshold) { return thold = threshold; }
    void restore();
    bool doProcess(samples_t time) override;
    void restart() override {}
    void restartScript() override { onOnsetHandler.store(nullptr); }
};

class OnOnsetClosure : public ProcessClosure {
    float onset;

  protected:
    void addParameters() {
        addFloat(onset);
        addTimeParameter();
    }

  public:
    OnOnsetClosure(ScriptFunction function, float onset, samples_t offset)
        : ProcessClosure(function, offset), onset(onset) {}
};

class OnsetDetectorFactory : public ProcessorCache<OnsetDetector> {
  public:
    static OnsetDetectorFactory &instance() {
        static OnsetDetectorFactory instance;
        return instance;
    }
    OnsetDetector *newOnsetDetector();
};

} // namespace audio
} // namespace bipscript

#endif // ONSETDETECTOR_H
