/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BOUNDARRAY_H
#define BOUNDARRAY_H

#include "virtualmachine.h"

#include <stdint.h>

namespace bipscript {

template <class T> class BoundArray {
    uint32_t len;
    T **array;
    HSQOBJECT sqarray;
    HSQUIRRELVM vm;

  public:
    BoundArray(uint32_t size) : len(size) {
        vm = VirtualMachine::instance().vm();
        array = new T *[size];
        sq_newarray(vm, size);
        // sq_resetobject(&sqarray); // initialize the handle
        sq_getstackobj(vm, -1, &sqarray);
        sq_addref(vm, &sqarray); // TODO: remove ref in dtor
        sq_poptop(vm);
    }
    ~BoundArray() { delete[] array; }
    void set(uint32_t index, T *elem, HSQOBJECT objectType) {
        array[index] = elem;
        sq_pushobject(vm, sqarray);
        sq_pushinteger(vm, index);
        sq_pushobject(vm, objectType);
        sq_createinstance(vm, -1);
        sq_setinstanceup(vm, -1, elem);
        sq_remove(vm, -2); // remove class object
        sq_set(vm, -3);
        sq_poptop(vm);
    }
    HSQOBJECT hsqobject() { return sqarray; }
    T &operator[](int index) { return *array[index]; } // for C++
};

} // namespace bipscript

#endif // BOUNDARRAY_H
