/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <chrono>
#include <thread>

#include "collectorthread.h"

namespace bipscript {

void *run_collector(void *arg) {
    ((CollectorThread *)arg)->run();
    return 0;
}

CollectorThread::CollectorThread() : queueList(512), finished(false) {
    int result = pthread_create(&thread, nullptr, run_collector, this);
    if (result) {
        throw std::logic_error("could not create collector thread");
    }
}

void CollectorThread::run() {
    while (true) {
        GarbageCollector *q = queueList.first();
        while (q) {
            q->free();
            q = queueList.next(q);
        }
        if (finished.load()) {
            break;
        }
        // TODO: how long?
        std::this_thread::sleep_for(std::chrono::milliseconds(5));
    }
}

} // namespace bipscript
