/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ITERATOR_H
#define ITERATOR_H

#include "bindutil.h"

#include <stdio.h>

namespace bipscript {

namespace binding {
extern HSQOBJECT iteratorObject;
}

class Iterator {
  public:
    static void bindType(HSQUIRRELVM vm) {
        BindUtil binding(vm);
        binding.startClass("iterator", 8, binding::iteratorObject);
        binding.addMethod("constructor", 11, Ctor);
        binding.addMethod("_tostring", 9, ToString);
        binding.addMethod("_nexti", 6, Nexti);
        binding.addMethod("_get", 4, Get);
        binding.endClass();
    }
    virtual ~Iterator() {}
    virtual bool hasNext() = 0;
    virtual void next(HSQUIRRELVM vm) = 0;

  private:
    // binding methods
#define BIND_CLASS_NAME "iterator"
#define BIND_METHOD_NAME "constructor"
    static SQInteger Ctor(HSQUIRRELVM vm) {
        return sq_throwerror(vm, "cannot directly instantiate iterator");
    }
#undef BIND_METHOD_NAME
#define BIND_METHOD_NAME "_tostring"
    static SQInteger ToString(HSQUIRRELVM vm) {
        // BIND_GET_THIS(Iterator, binding::iteratorObject)
        sq_pushstring(vm, "(iterator)", -1);
        return 1;
    }
#undef BIND_METHOD_NAME
#define BIND_METHOD_NAME "_nexti"
    static SQInteger Nexti(HSQUIRRELVM vm) {
        BIND_GET_THIS_PROPERTY(Iterator)
        if (obj->hasNext()) {
            sq_pushinteger(vm, 0); // push something else?
            return 1;
        }
        sq_pushnull(vm);
        return 1;
    }
#undef BIND_METHOD_NAME
#define BIND_METHOD_NAME "_get"
    static SQInteger Get(HSQUIRRELVM vm) {
        SQObjectType argtype = sq_gettype(vm, 2);
        if (argtype == OT_INTEGER) {
            SQInteger index;
            sq_getinteger(vm, 2, &index);
            BIND_GET_THIS_PROPERTY(Iterator)
            if (index == 0) {
                obj->next(vm);
                return 1;
            }
        }
        sq_pushnull(vm);
        return sq_throwobject(vm);
    }
#undef BIND_METHOD_NAME
#undef BIND_CLASS_NAME
};

} // namespace bipscript

#endif // ITERATOR_H
