/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef JACKCLOCK_H
#define JACKCLOCK_H

#include "clock.h"
#include "clockcollector.h"
#include "jacktransport.h"

namespace bipscript {
namespace jack {

class Clock : public time::Clock {
    Transport transport;

  public:
    Clock(jack_client_t *client) : time::Clock(120), transport(client) {}
    void start() override { transport.start(); }
    void stop() override { transport.stop(); }
    void update(samples_t) override;
    bool hasHistory() override { return false; }
    void recycle() override { time::ClockCollector::instance().recycle(this); }
};

class ClockFactory {
  public:
    static ClockFactory &instance() {
        static ClockFactory instance;
        return instance;
    }
    Clock *newClock();
};

} // namespace jack
} // namespace bipscript

#endif // JACKCLOCK_H
