/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef LISTABLE_H
#define LISTABLE_H

namespace bipscript {

/**
 * Superclass for all classes stored on a linked list
 */
template <class T> class Listable {
  public:
    T *mnext;
    Listable() : mnext(nullptr) {}
};

/**
 * Linked list implementation
 */
template <class T> class List {
  protected:
    T *mfirst;
    T *mlast;

  public:
    List() : mfirst(nullptr) {}

    void add(T *elem) {
        if (!mfirst) {
            mlast = elem;
        }
        elem->mnext = mfirst;
        mfirst = elem;
    }
    void addAll(List &list) {
        if (!mfirst) {
            mfirst = list.mfirst;
            mlast = list.mlast;
        } else {
            mlast->mnext = list.mfirst;
            mlast = list.mlast;
        }
    }
    void insert(T *elem) {
        if (!mfirst) {
            mfirst = mlast = elem;
        } else if (*mlast <= *elem) {
            mlast->mnext = elem;
            mlast = elem;
        } else if (*elem < *mfirst) {
            elem->mnext = mfirst;
            mfirst = elem;
        } else {
            T *e = mfirst;
            while (*(e->mnext) <= *elem) {
                e = e->mnext;
            }
            elem->mnext = e->mnext;
            e->mnext = elem;
        }
    }

    T *first() { return mfirst; }
    T *next(T *elem) { return elem->mnext; }
    T *pop() { return mfirst = mfirst->mnext; }

    bool remove(T *elem) {
        if (elem == mfirst) {
            mfirst = mfirst->mnext;
            return true;
        } else {
            T *l = mfirst;
            while (l) {
                if (l->mnext == elem) {
                    l->mnext = elem->mnext;
                    if (mlast == elem) {
                        mlast = l;
                    }
                    return true;
                }
                l = l->mnext;
            }
        }
        return false;
    }
    T *clear() {
        T *ret = mfirst;
        mfirst = nullptr;
        return ret;
    }

    // basic bubble sort
    T *sort() {
        bool swapped;
        T *end = nullptr;
        do {
            T *last, *ptr = mfirst;
            swapped = false;
            while (ptr && ptr->mnext != end) {
                T *next = ptr->mnext;
                if (*ptr > *next) {
                    if (ptr == mfirst) {
                        mfirst = next;
                    } else {
                        last->mnext = next;
                    }
                    ptr->mnext = next->mnext;
                    next->mnext = ptr;
                    last = next;
                    swapped = true;
                } else {
                    last = ptr;
                    ptr = next;
                }
            }
            end = ptr;
        } while (swapped);
        return mfirst;
    }
};

} // namespace bipscript

#endif // LISTABLE_H
