/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "methodqueue.h"

namespace bipscript {

MethodQueue::MethodQueue() : dispatchQueue(512), mactive(0) {}

void MethodQueue::dispatch(Closure *function) {
    mactive++;
    // don't do this in process thread!
    // TODO: make a waiting queue instead
    while (!dispatchQueue.push(function)) {
    }
}

Closure *MethodQueue::next() {
    Closure *function;
    if (dispatchQueue.pop(function)) {
        return function;
    }
    return nullptr;
}

void MethodQueue::finished(Closure *closure) {
    closure->finished();
    mactive--;
}

void MethodQueue::scriptReset() {
    // drain queue and recycle objects
    Closure *closure = next();
    while (closure) {
        closure->finished();
        closure = next();
    }
    mactive.store(0);
}

} // namespace bipscript
