/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "midibeatclock.h"

#include <math.h>

namespace bipscript {
namespace midi {

bool ClockGenerator::doProcess(samples_t time) {
    connection.reset();
    bool running = mclock.timer().running();
    if (running) {
        time::Window *window = mclock.timer().firstpos();
        if (!rolling) {
            connection.addStart(window->frame() == 0);
        }
        // get samplerate, does not change over period
        samples_t sampleRate = window->sampleRate();
        while (window) {
            float bpm = window->bpm();
            // 24 pulses per quarter = how many frames
            double frameDelta = (sampleRate * 60) / (24 * bpm);
            // start the counter staggered from the last window
            double counter = rolling ? frameDelta * remaining : 0.0;
            // loop over pulses in this window
            uint32_t nextPulse = lround(counter);
            while (nextPulse < window->size()) {
                connection.addPulse(nextPulse);
                counter += frameDelta;
                nextPulse = lround(counter);
            }
            // remaining size of the last chopped-off pulse
            remaining = (counter - window->size()) / frameDelta;
            window = mclock.timer().nextpos(window);
        }
    } else if (rolling) {
        connection.addStop();
    }
    midi::Source::reportLevels(this, time);
    rolling = running;
    // if the clock is running we have beat events queued so technically still active
    return running;
}

ClockGeneratorConnection::~ClockGeneratorConnection() {}

} // namespace midi
} // namespace bipscript
