/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef MIDIPATTERN_H
#define MIDIPATTERN_H

#include <vector>

#include "closure.h"
#include "midievent.h"

namespace bipscript {
namespace midi {

class PatternFilter : public ScriptFunction {
  public:
    PatternFilter(ScriptFunction &function);
    bool execute(size_t index, NoteEvent *event);
};

class Sequence;

class Pattern {
    std::vector<NoteEvent *> noteList;

  public:
    Pattern() {}
    Pattern(Pattern &copy);
    ~Pattern();
    void add(Note &note, float measure);
    void add(Note *note, float measure) { add(*note, measure); }
    void add(Pattern &pattern, float measure = 1.0);
    uint32_t len() { return noteList.size(); }
    NoteEvent *operator[](int32_t index) {
        return index < static_cast<int>(noteList.size()) ? noteList[index] : nullptr;
    }
    std::string print();
    Pattern *filter(ScriptFunction func);
    Pattern *transpose(int amount);
    Pattern *slice(float start, float end);
    Sequence *sequence();
};

} // namespace midi
} // namespace bipscript

#endif // MIDIPATTERN_H
