/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mml.h"
#include "clock.h"
#include "midifile.h"
#include "module_midi.h"

namespace bipscript {
namespace midi {

static void mml_callback(MML_INFO *p, void *extobj) {
    MML *mmlReader = static_cast<MML *>(extobj);
    switch (p->type) {
    case MML_TYPE_NOTE:
        mmlReader->processNote(p);
        break;
    case MML_TYPE_REST:
        mmlReader->processRest(p);
        break;
    case MML_TYPE_TEMPO:
        mmlReader->tempo(p);
        break;
    case MML_TYPE_LENGTH:
        break;
    case MML_TYPE_OCTAVE:
        break;
    case MML_TYPE_VOLUME:
        mmlReader->volume(p);
        break;
    case MML_TYPE_OCTAVE_UP:
        break;
    case MML_TYPE_OCTAVE_DOWN:
        break;
    case MML_TYPE_USER_EVENT:
        break;
    }
}

MML::MML() : mvolume(0), chordSize(0) {
    mml_init(&mml, mml_callback, this);
    MML_OPTION_INITIALIZER_DEFAULT(&mmlopt);
}

void MML::processNote(MML_INFO *p) {
    MML_ARGS_NOTE *mmlNote = &(p->args.note);
    // printf("note number %d ticks %d/%f\n", mmlNote->number, mmlNote->ticks, ticksPerBar);
    int velocity = mvolume ? mvolume : 64;
    if (mmlNote->ticks) {
        float duration = mmlNote->ticks / ticksPerBar;
        // add waiting chord if one
        for (int i = 0; i < chordSize; i++) {
            Note *chordNote =
                new Note(chordTone[i], velocity, duration); // TODO: velocity as parameter
            msequence->add(*chordNote, mmeasure);
        }
        Note *note = new Note(mmlNote->number, velocity, duration); // TODO: velocity as parameter
        msequence->add(*note, mmeasure);
        mmeasure += duration;
        // reset chord
        chordSize = 0;
    } else {
        // zero length note is part of a chord
        if (chordSize < MAX_CHORD_SIZE) {
            chordTone[chordSize++] = mmlNote->number;
        }
    }
}

void MML::processRest(MML_INFO *p) {
    MML_ARGS_REST *note = &(p->args.rest);
    mmeasure += note->ticks / ticksPerBar;
    // reset chord
    chordSize = 0;
}

void MML::tempo(MML_INFO *p) {
    MML_ARGS_TEMPO tempo = p->args.tempo;
    msequence->add(*(new TempoChange(tempo.value)), mmeasure);
}

void MML::volume(MML_INFO *p) { mvolume = p->args.volume.value; }

Sequence *MML::sequence(const char *text) {
    msequence = new Sequence();
    mmeasure = 1.0;
    ticksPerBar = mmlopt.bticks * 4.0f;
    mml_setup(&mml, &mmlopt, (char *)text);
    while (mml_fetch(&mml) == MML_RESULT_OK) {
    }
    return msequence;
}

} // namespace midi
} // namespace bipscript
