/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef TAB_H
#define TAB_H

#include <map>

#include "midisequence.h"

namespace bipscript {
namespace midi {

typedef std::map<char, uint8_t> velocitymap;

class TabBar {
    int8_t *notes;
    double mstart;
    int mopen;

  public:
    static int division;
    TabBar(float start, bool open) : mstart(start), mopen(open ? 0 : -1) {
        notes = new int8_t[division];
        for (int i = 0; i < division; i++) {
            notes[i] = 0;
        }
    }
    ~TabBar() { delete[] notes; }
    float start() { return mstart; }
    void add(uint8_t vel, float measure);
    void end(float measure);
    bool finalize();
    int size() { return division; }
    std::string print(velocitymap &vmap);

  private:
    int quantize(float meaure);
    char closest(uint8_t velocity, velocitymap &vmap);
};

class TabLine {
    uint8_t mnote;
    TabBar *current;
    std::vector<TabBar *> mbars;

  public:
    TabLine(uint8_t note) : mnote(note) {
        current = new TabBar(1.0, false);
        mbars.push_back(current);
    }
    ~TabLine();
    void add(uint8_t vel, float measure);
    void end(float measure);
    void forward(float measure);
    uint8_t note() { return mnote; }
    std::string print(uint bar, velocitymap &vmap);
    std::vector<TabBar *> &bars() { return mbars; }
};

struct Strikedef {
    uint8_t pitch;
    uint8_t velocity;
    Strikedef(uint8_t p, uint8_t v) : pitch(p), velocity(v) {}
};

typedef std::map<char, Strikedef *> strikemap;

class Tab {
    uint mlinewrap;
    std::map<std::string, strikemap> prefixmap;
    velocitymap vmap;

  public:
    Tab() : mlinewrap(80) {}
    ~Tab();
    std::string string(midi::Pattern &pattern);
    std::string string(midi::Sequence &seq);
    midi::Sequence *sequence(const char *tab);
    // customization methods
    void define(std::string prefix, std::string strike, uint8_t pitch, uint8_t velocity);
    void velocity(std::string strike, uint8_t velocity);
    // properties
    int linewrap() { return mlinewrap; }
    void linewrap(int value);
    int division() { return TabBar::division; }
    void division(int value);

  private:
    static bool linecmp(TabLine *l1, TabLine *l2);
    NoteOn *readbar(std::string prefix, int pitch, NoteOn *lastnote, float start, std::string &,
                    Sequence &seq);
};

} // namespace midi
} // namespace bipscript

#endif // TAB_H
