/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PRINTFUNC_H
#define PRINTFUNC_H

#include <stdarg.h>
#include <stdio.h>

#include "bindutil.h"
#include "squirrel.h"

namespace bipscript {

// adds print methods to existing global context
class PrintFunc {

  public:
    static void bindType(HSQUIRRELVM vm) {
        BindUtil binding(vm);
        // push root table
        sq_pushroottable(vm);
        // add methods
        binding.addMethod("println", 7, Println);
        // pop root table
        sq_poptop(vm);
    }

    static void Print(HSQUIRRELVM, const SQChar *format, ...) {
        va_list args;
        va_start(args, format);
        vfprintf(stdout, format, args);
        va_end(args);
    }

    static SQInteger Println(HSQUIRRELVM vm) {
        if (sq_gettop(vm) != 2) {
            return sq_throwerror(vm, "println method expected exactly 1 parameter");
        }
        const SQChar *str;
        if (SQ_SUCCEEDED(sq_tostring(vm, 2)) && SQ_SUCCEEDED(sq_getstring(vm, -1, &str))) {
            printf("%s\n", str);
            return SQ_OK;
        }
        return SQ_ERROR;
    }
};

} // namespace bipscript

#endif // PRINTFUNC_H
