/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef EVENTCLOSURE_H
#define EVENTCLOSURE_H

#include "clock.h"
#include "closure.h"

namespace bipscript {
/**
 * represents a closure created from the process thread using a memory pool
 *
 */
class ProcessClosure : public Closure, public Listable<ProcessClosure> {
    time::Timer &mclock;
    samples_t offset; // event offset in current period
    static Collector<ProcessClosure> collector;

  protected:
    void addTimeParameter() { addFloat(mclock.offsetToMeasure(offset)); }

  public:
    static void free() { collector.free(); }
    ProcessClosure(ScriptFunction &function, samples_t offset)
        : Closure(function, nullptr), mclock(time::Clock::defaultClock().timer()), offset(offset) {}
    void *operator new(size_t size) { return MemoryPool::processPool().malloc(size); }
    void operator delete(void *p) { MemoryPool::processPool().free(p); }
    virtual void finished() { collector.recycle(this); }
};

} // namespace bipscript

#endif // EVENTCLOSURE_H
