/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef QUEUELIST_H
#define QUEUELIST_H

#include "listable.h"
#include "threadqueue.h"

namespace bipscript {

/**
 * A linked list that pulls from a single producer single consumer queue. One thread
 * adds items that can be iterated by a separate thread. Items are not sorted.
 */
template <class T> class QueueList {
    List<T> list;
    ThreadQueue<T> queue;

  public:
    QueueList(std::size_t size) : queue(size) {}
    void add(T *elem) { queue.add(elem); }
    T *first() {
        T *fresh;
        while (queue.pop(fresh)) {
            list.add(fresh);
        }
        return list.first();
    }
    T *next(T *elem) { return list.next(elem); }
    void remove(T *elem) { list.remove(elem); }
    void clear() {
        T *fresh;
        while (queue.pop(fresh)) {
        }
        list.clear();
    }
    T *sort() { return list.sort(); }
};

} // namespace bipscript
#endif // QUEUELIST_H
