﻿/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SCRIPTARRAY_H
#define SCRIPTARRAY_H

#include "bindutil.h"
#include "virtualmachine.h"

#include <boost/type_index.hpp>
#include <stdexcept>
#include <stdint.h>

namespace bipscript {

template <class T> class ScriptArray {
    HSQOBJECT sqarray;
    HSQOBJECT &mtype; // element type
    HSQUIRRELVM vm;

  public:
    ScriptArray(uint32_t size, HSQOBJECT &type) : mtype(type) {
        vm = VirtualMachine::instance().vm();
        sq_newarray(vm, size);
        // sq_resetobject(&sqarray); // initialize the handle
        sq_getstackobj(vm, -1, &sqarray);
        sq_addref(vm, &sqarray);
        sq_poptop(vm);
    }
    ScriptArray(HSQOBJECT &object, HSQOBJECT &type) : sqarray(object), mtype(type) {
        vm = VirtualMachine::instance().vm();
        sq_addref(vm, &sqarray);
    }
    ~ScriptArray() { sq_release(vm, &sqarray); }
    void set(uint32_t index, T *elem) {
        sq_pushobject(vm, sqarray);
        sq_pushinteger(vm, index);
        sq_pushobject(vm, mtype);
        sq_createinstance(vm, -1);
        sq_setinstanceup(vm, -1, elem);
        sq_setreleasehook(vm, -1, BindUtil::Delete<T>); // TODO: make an option
        sq_remove(vm, -2);                              // remove class object
        sq_set(vm, -3);
        sq_poptop(vm);
    }
    T &operator[](int index) const { // for C++
        sq_pushobject(vm, sqarray);
        sq_pushinteger(vm, index);
        sq_get(vm, -2);
        SQUserPointer objPtr;
        if (SQ_FAILED(sq_getinstanceup(vm, -1, &objPtr, (SQUserPointer)&mtype, false))) {
            sq_pop(vm, 2);
            throw std::logic_error(std::string("Array member is not of type ") +
                                   boost::typeindex::type_id<T>().pretty_name());
        }
        sq_pop(vm, 2); // pop element + array
        return *(static_cast<T *>(objPtr));
    }
    HSQOBJECT hsqobject() { return sqarray; }
    uint32_t len() {
        sq_pushobject(vm, sqarray);
        SQInteger ret = sq_getsize(vm, -1);
        sq_poptop(vm);
        return ret;
    }
};

} // namespace bipscript

#endif // SCRIPTARRAY_H
