/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SENDQUEUE_H
#define SENDQUEUE_H

#include "collectorthread.h"
#include "threadqueue.h"

namespace bipscript {

/**
 * Combination of threadqueue and collector for "send()" events
 */
template <class T> class SendQueue {
    ThreadQueue<T> mqueue;
    List<T> mdone;
    Collector<T> mcollector;

  public:
    SendQueue(size_t size) : mqueue(size) { CollectorThread::instance().addCollector(&mcollector); }
    void add(T *elem) { mqueue.add(elem); }
    bool pop(T *&elem) {
        bool ret = mqueue.pop(elem);
        if (ret) {
            mdone.add(elem);
        }
        return ret;
    }
    void update() {
        T *evt = mdone.first();
        while (evt) {
            T *done = evt;
            evt = mdone.pop();
            mcollector.recycle(done); // TODO: send list instead of one at a time
        }
    }
};

} // namespace bipscript

#endif // SENDQUEUE_H
