/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SORTEDQUEUELIST_H
#define SORTEDQUEUELIST_H

#include "listable.h"
#include "threadqueue.h"
// TODO: should be a factor period size
// and also size of existing list
#define SQ_UPDATE_MAX_EVENTS 32

namespace bipscript {

/**
 * An event list that pulls from a single producer single consumer queue. One thread
 * adds items that can be iterated by a separate thread. Items are always in order.
 */
template <class T> class SortedQueueList {
    ThreadQueue<T> eventQueue; // script thread -> process thread
    List<T> sortedEvents;      // local to process thread
  public:
    SortedQueueList() : eventQueue(2048) {}
    void addEvent(T *evt);
    T *firstEvent() { return sortedEvents.first(); }
    T *nextEvent(T *current) { return sortedEvents.next(current); }
    T *pop() { return sortedEvents.pop(); }
    int update();
};

// runs in script thread
template <class T> void SortedQueueList<T>::addEvent(T *evt) { eventQueue.add(evt); }

// process thread
template <class T> int SortedQueueList<T>::update() {
    T *freshEvent;
    int counter = 0;
    while (counter < SQ_UPDATE_MAX_EVENTS && eventQueue.pop(freshEvent)) {
        sortedEvents.insert(freshEvent);
        counter++;
    }
    return counter;
}

} // namespace bipscript
#endif // SORTEDQUEUELIST_H
