/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "basicclock.h"

namespace bipscript {
namespace time {

void BasicClock::signature(Signature *signature) {
    if (!mrunning) {
        msignature = *signature;
        commands.add(new SignatureChange(signature));
    } else {
        throw std::logic_error("cannot update signature while clock is running");
    }
}

void BasicClock::clockChanges() {
    ClockChange *command;
    bool changed = false;
    while (commands.pop(command)) {
        if (command->type() == ClockChange::Type::RESTART) {
            following.frame(0);
            following.measure(1.0);
            following.reposition(true);
            Clock::running(true);
        } else if (command->type() == ClockChange::Type::STATE) {
            Clock::running(static_cast<StateChange *>(command)->start());
        } else if (command->type() == ClockChange::Type::SIGNATURE) {
            following.signature(static_cast<SignatureChange *>(command)->signature());
        } else if (command->type() == ClockChange::Type::TEMPO) {
            following.bpm(static_cast<TempoChange *>(command)->tempo());
        }
        ClockCollector::instance().recycle(command);
        changed = true;
    }
    if (changed) {
        StatusServer &statusserver = StatusServer::instance();
        if (statusserver.active()) {
            float measure = following.measure();
            uint16_t num = msignature.numerator();
            uint16_t den = msignature.denominator();
            statusserver.clockUpdate(this, mtimer.running(), measure, following.bpm(), num, den);
        }
    }
}

void BasicClock::update(samples_t) {
    clockChanges();
    if (mtimer.running()) {
        StatusServer &statusserver = StatusServer::instance();
        if (statusserver.active()) {
            uint32_t bar = static_cast<uint32_t>(following.measure());
            if (bar != static_cast<uint32_t>(current.measure())) {
                statusserver.clockTime(this, following.measure());
            }
        }
        samples_t range = period();
        // update current
        current = following;
        current.size(range);
        mtempo.store(current.bpm());
        // compute following
        following.frame(current.frame() + range);
        following.measure(current.measure() + current.moffset(range));
        following.reposition(false);
        updateRemotes();
    }
}

} // namespace time
} // namespace bipscript
