﻿/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CLOCKCHANGEEVENT_H
#define CLOCKCHANGEEVENT_H

#include "clock.h"
#include "event.h"

namespace bipscript {
namespace time {

/**
 * Scheduled clock change, e.g. tempo, relocation.
 */
class ClockChangeEvent : public BaseEvent, public Listable<ClockChangeEvent> {
  protected:
    Window mwindow;

  public:
    enum TimeChangeEventType { SIGNATURE, TEMPO, RELOCATION };
    static ClockChangeEvent *tempoChangeEvent(float bpm, float measure) {
        ClockChangeEvent *ret = new ClockChangeEvent(measure, TEMPO);
        ret->mwindow.bpm(bpm);
        ret->mwindow.reposition(false);
        return ret;
    }
    static ClockChangeEvent *timeChangeEvent(Signature &signature, float measure) {
        ClockChangeEvent *ret = new ClockChangeEvent(measure, SIGNATURE);
        ret->mwindow.signature(signature);
        ret->mwindow.reposition(false);
        return ret;
    }
    static ClockChangeEvent *relocationEvent(float location, float measure) {
        ClockChangeEvent *ret = new ClockChangeEvent(measure, RELOCATION);
        ret->mwindow.measure(static_cast<double>(location));
        ret->mwindow.reposition(true);
        return ret;
    }
    Window *window() { return &mwindow; }
    TimeChangeEventType type() { return mtype; }

  private:
    ClockChangeEvent(float measure, TimeChangeEventType type) : BaseEvent(measure), mtype(type) {
        mwindow.measure(static_cast<double>(measure));
    }
    TimeChangeEventType mtype;
};

} // namespace time
} // namespace bipscript

#endif // CLOCKCHANGEEVENT_H
