/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CLOCKCOLLECTOR_H
#define CLOCKCOLLECTOR_H

#include "clock.h"
#include "clockchangeevent.h"
#include "collectorthread.h"

namespace bipscript {
namespace time {

/**
 * Garbage collector for clocks, timers and clock commands.
 */
class ClockCollector {
    Collector<Clock> clockCollector;
    Collector<RemoteTimer> timerCollector;
    Collector<ClockChange> commandCollector;
    Collector<ClockChangeEvent> eventCollector;
    ClockCollector() {
        CollectorThread::instance().addCollector(&clockCollector);
        CollectorThread::instance().addCollector(&timerCollector);
        CollectorThread::instance().addCollector(&commandCollector);
        CollectorThread::instance().addCollector(&eventCollector);
    }

  public:
    static ClockCollector &instance() {
        static ClockCollector instance;
        return instance;
    }
    void recycle(Clock *clock) { clockCollector.recycle(clock); }
    void recycle(RemoteTimer *timer) { timerCollector.recycle(timer); }
    void recycle(ClockChange *command) { commandCollector.recycle(command); }
    void recycle(ClockChangeEvent *event) { eventCollector.recycle(event); }
};

} // namespace time
} // namespace bipscript

#endif // CLOCKCOLLECTOR_H
