/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef VIRTUALMACHINE_H
#define VIRTUALMACHINE_H

#include "squirrel.h"
#include <cstdio>

class VirtualMachine {
    HSQUIRRELVM mvm;

  public:
    static VirtualMachine &instance() {
        static VirtualMachine instance;
        return instance;
    }
    HSQUIRRELVM vm() { return mvm; }
    void vm(HSQUIRRELVM vm) { mvm = vm; }

    static void printvalue(HSQUIRRELVM vm, int n) {
        switch (sq_gettype(vm, n)) {
        case OT_NULL:
            printf("null ");
            break;
        case OT_INTEGER:
            SQInteger val;
            sq_getinteger(vm, n, &val);
            printf("int(%lld) ", val);
            break;
        case OT_FLOAT:
            printf("float ");
            break;
        case OT_STRING:
            const SQChar *str;
            sq_getstring(vm, n, &str);
            printf("string(%s) ", str);
            break;
        case OT_TABLE:
            printf("table ");
            break;
        case OT_ARRAY:
            printf("array ");
            break;
        case OT_USERDATA:
            printf("userdata ");
            break;
        case OT_CLOSURE:
            printf("closure ");
            break;
        case OT_NATIVECLOSURE:
            printf("native closure ");
            break;
        case OT_GENERATOR:
            printf("generator ");
            break;
        case OT_USERPOINTER:
            printf("userpointer ");
            break;
        case OT_CLASS:
            printf("class ");
            break;
        case OT_INSTANCE:
            printf("instance ");
            break;
        case OT_WEAKREF:
            printf("weak ref ");
            break;
        default:
            break;
        }
    }
    static void printstack(HSQUIRRELVM vm, const char *message, int startdepth = 1) {
        printf("%s:\t", message);
        printstack(vm, startdepth);
    }

    static void printstack(HSQUIRRELVM vm, int startdepth = 1) {
        SQInteger nargs = sq_gettop(vm);
        for (SQInteger n = startdepth; n <= nargs; n++) {
            printf("%lld. ", n);
            printvalue(vm, n);
        }
        printf("\n");
    }
};

#endif // VIRTUALMACHINE_H
