/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef WRITER_H
#define WRITER_H

#include "bindutil.h"
#include <stddef.h>
#include <stdexcept>

namespace bipscript {

namespace binding {
extern HSQOBJECT writerObject;
extern HSQOBJECT bufferObject;
} // namespace binding

class Buffer;

class Writer {

  public:
    static void bindType(HSQUIRRELVM vm) {
        BindUtil binding(vm);
        binding.startClass("writer", 5, binding::writerObject);
        binding.addMethod("_cloned", 7, NoClone);
        binding.addMethod("_tostring", 9, ToString);
        binding.addMethod("print", 5, Print);
        binding.addMethod("println", 7, Println);
        binding.addMethod("write", 5, Write);
        binding.endClass();
    }

    //    static Writer *getWriter(HSQUIRRELVM vm, SQInteger index) {
    //        return BindUtil::getObject<Writer>(vm, index, binding::writerObject);
    //    }

    virtual ~Writer() {}
    virtual void print(const char *) = 0;
    virtual void println(const char *) = 0;
    virtual void write(bool b) = 0;
    virtual void write(const char *) = 0;
    virtual void write(float f) = 0;
    virtual void write(long i) = 0;
    virtual void write(Buffer *) = 0;

  private:
    // binding methods
#define BIND_CLASS_NAME "writer"
    static SQInteger NoClone(HSQUIRRELVM vm) {
        return sq_throwerror(vm, "cannot clone objects of type " BIND_CLASS_NAME);
    }
#define BIND_METHOD_NAME "_tostring"
    static SQInteger ToString(HSQUIRRELVM vm) {
        sq_pushstring(vm, "(writer)", -1);
        return 1;
    }
#undef BIND_METHOD_NAME
#define BIND_METHOD_NAME "print"
    static SQInteger Print(HSQUIRRELVM vm) {
        BIND_GET_NARGS()
        BIND_NARGS_CHECK_EXACT(1)
        BIND_GET_THIS(Writer, binding::writerObject)
        const SQChar *str;
        if (SQ_SUCCEEDED(sq_tostring(vm, 2)) && SQ_SUCCEEDED(sq_getstring(vm, -1, &str))) {
            BIND_WRAP_TRY(obj->print(str));
            return SQ_OK;
        }
        return SQ_ERROR;
    }
#undef BIND_METHOD_NAME
#define BIND_METHOD_NAME "println"
    static SQInteger Println(HSQUIRRELVM vm) {
        BIND_GET_NARGS()
        BIND_NARGS_CHECK_EXACT(1)
        BIND_GET_THIS(Writer, binding::writerObject)
        const SQChar *str;
        if (SQ_SUCCEEDED(sq_tostring(vm, 2)) && SQ_SUCCEEDED(sq_getstring(vm, -1, &str))) {
            BIND_WRAP_TRY(obj->println(str));
            return SQ_OK;
        }
        return SQ_ERROR;
    }
#undef BIND_METHOD_NAME
#define BIND_METHOD_NAME "write"
    static SQInteger Write(HSQUIRRELVM vm) {
        BIND_GET_NARGS()
        BIND_NARGS_CHECK_EXACT(1)
        BIND_GET_THIS(Writer, binding::writerObject)
        SQObjectType argtype = sq_gettype(vm, 2);
        if (argtype == OT_FLOAT) {
            SQFloat value;
            sq_getfloat(vm, 2, &value);
            BIND_WRAP_TRY(obj->write(value));
        } else if (argtype == OT_INTEGER) {
            SQInteger value;
            sq_getinteger(vm, 2, &value);
            BIND_WRAP_TRY(obj->write((long)value));
        } else if (argtype == OT_BOOL) {
            SQBool value;
            sq_getbool(vm, 2, &value);
            BIND_WRAP_TRY(obj->write((bool)value));
        } else if (argtype == OT_STRING) {
            const SQChar *value;
            sq_getstring(vm, 2, &value);
            BIND_WRAP_TRY(obj->write(value));
        } else if (argtype == OT_INSTANCE) {
            SQUserPointer typetag;
            sq_gettypetag(vm, 2, &typetag);
            if (typetag == &binding::bufferObject) {
                bipscript::Buffer *value =
                    BindUtil::getObject<bipscript::Buffer>(vm, 2, binding::bufferObject);
                BIND_WRAP_TRY(obj->write(value));
            } else {
                return sq_throwerror(vm, "writer unsupported type");
            }
        } else {
            return sq_throwerror(vm, "writer unsupported type TBD");
        }
        return 1;
    }
#undef BIND_METHOD_NAME
#undef BIND_CLASS_NAME
};

} // namespace bipscript

#endif // WRITER_H
